@@ -73,7 +73,7 @@ ControlView::ControlView( QWidget *parent )
     m_splitter->setStretchFactor( m_splitter->indexOf( m_marbleWidget ), 1 );
     m_splitter->setSizes( QList<int>() << 180 << width() - 180 );
 
-    m_control->addMarbleWidget( m_marbleWidget );
+    m_control->setMarbleWidget( m_marbleWidget );
 
     connect( m_control, SIGNAL( showMapWizard() ), this, SIGNAL( showMapWizard() ) );
     connect( m_control, SIGNAL( showUploadDialog() ), this, SIGNAL( showUploadDialog() ) );
@@ -86,7 +86,7 @@ ControlView::~ControlView()
 
 QString ControlView::applicationVersion()
 {
-    return "1.1.90 (1.2 RC 1)";
+    return "1.2.85 (1.3 Beta 2)";
 }
 
 void ControlView::zoomIn()
@@ -163,11 +163,7 @@ QString ControlView::defaultMapThemeId() const
       fallBackThemes << "earth/openstreetmap/openstreetmap.dgml";
     }
 
-    QStringList installedThemes;
-    QList<GeoSceneDocument const*> themes = m_marbleWidget->model()->mapThemeManager()->mapThemes();
-    foreach(GeoSceneDocument const* theme, themes) {
-        installedThemes << theme->head()->mapThemeId();
-    }
+    const QStringList installedThemes = m_marbleWidget->model()->mapThemeManager()->mapThemeIds();
 
     foreach(const QString &fallback, fallBackThemes) {
         if (installedThemes.contains(fallback)) {
@@ -207,7 +203,7 @@ void ControlView::printMapScreenShot( QPointer<QPrintDialog> printDialog)
                 // Temporarily remove the black background and layers painting on it
                 m_marbleWidget->setShowBackground( false );
                 m_marbleWidget->setPalette( QPalette ( Qt::white ) );
-                m_marbleWidget->repaint();
+                m_marbleWidget->update();
             }
 
             if ( printOptions->printMap() ) {
@@ -237,7 +233,7 @@ void ControlView::printMapScreenShot( QPointer<QPrintDialog> printDialog)
             if ( hideBackground ) {
                 m_marbleWidget->setShowBackground( wasBackgroundVisible );
                 m_marbleWidget->setPalette( originalPalette );
-                m_marbleWidget->repaint();
+                m_marbleWidget->update();
             }
     }
 #endif
@@ -538,11 +534,6 @@ void ControlView::addGeoDataFile( QString filename )
     m_marbleWidget->model()->addGeoDataFile( filename );
 }
 
-SunLocator* ControlView::sunLocator()
-{
-    return m_marbleWidget->model()->sunLocator();
-}
-
 }
 
 #include "ControlView.moc"
