@@ -57,7 +57,6 @@
 #include "AbstractFloatItem.h"
 #include "MarbleModel.h"
 #include "MarbleClock.h"
-#include "SunLocator.h"
 #include "BookmarkManager.h"
 #include "NewBookmarkFolderDialog.h"
 #include "GeoDataPlacemark.h"
@@ -89,7 +88,7 @@ namespace
 using namespace Marble;
 /* TRANSLATOR Marble::MainWindow */
 
-MainWindow::MainWindow(const QString& marbleDataPath, QWidget *parent) :
+MainWindow::MainWindow(const QString& marbleDataPath, const QVariantMap& cmdLineSettings, QWidget *parent) :
         QMainWindow(parent),
         m_sunControlDialog( 0 ),
         m_timeControlDialog( 0 ),
@@ -142,14 +141,16 @@ MainWindow::MainWindow(const QString& marbleDataPath, QWidget *parent) :
     m_position = NOT_AVAILABLE;
     m_distance = marbleWidget()->distanceString();
     m_clock = QLocale().toString( m_controlView->marbleModel()->clockDateTime().addSecs( m_controlView->marbleModel()->clockTimezone() ), QLocale::ShortFormat );
-    QTimer::singleShot( 0, this, SLOT( initObject() ) );
+    QMetaObject::invokeMethod(this,
+                              "initObject", Qt::QueuedConnection,
+                              Q_ARG(QVariantMap, cmdLineSettings));
 }
 
-void MainWindow::initObject()
+void MainWindow::initObject(const QVariantMap& cmdLineSettings)
 {
     QCoreApplication::processEvents ();
     setupStatusBar();
-    readSettings();
+    readSettings(cmdLineSettings);
 }
 
 void MainWindow::createActions()
@@ -468,9 +469,11 @@ void MainWindow::createBookmarksListMenu( QMenu *m_bookmarksListMenu, const GeoD
 
         GeoDataLookAt* lookAt = (*i)->lookAt();
         if ( !lookAt ) {
+            GeoDataCoordinates coordinates = (*i)->coordinate( m_controlView->marbleModel()->clockDateTime() );
             GeoDataLookAt coordinateToLookAt;
-            coordinateToLookAt.setCoordinates( (*i)->coordinate() );
-            coordinateToLookAt.setRange( (*i)->coordinate().altitude() );
+            coordinateToLookAt.setCoordinates( coordinates );
+            coordinateToLookAt.setRange( coordinates.altitude() );
+            var.setValue( coordinateToLookAt );
         } else {
             var.setValue( *lookAt );
         }
@@ -706,9 +709,6 @@ void MainWindow::workOffline( bool offline )
     m_controlView->marbleControl()->setWorkOffline( offline );
 
     m_workOfflineAct->setChecked( offline ); // Sync state with the GUI
-    if ( m_routingWidget ) {
-        m_routingWidget->setWorkOffline( offline );
-    }
 }
 
 void MainWindow::showAtmosphere( bool isChecked )
@@ -735,7 +735,7 @@ void MainWindow::lockPosition( bool isChecked )
 void MainWindow::controlSun()
 {
     if (!m_sunControlDialog) {
-        m_sunControlDialog = new SunControlWidget( m_controlView->sunLocator() );
+        m_sunControlDialog = new SunControlWidget( m_controlView->marbleWidget(), this );
         connect( m_sunControlDialog, SIGNAL( showSun( bool ) ),
                  this,               SLOT ( showSun( bool ) ) );
     }
@@ -760,7 +760,7 @@ void MainWindow::controlTime()
 
 void MainWindow::showSun( bool active )
 {
-    m_controlView->sunLocator()->setShow( active );
+    m_controlView->marbleWidget()->setShowSunShading( active );
 }
 
 void MainWindow::reload()
@@ -829,8 +829,13 @@ void MainWindow::updateStatusBar()
 void MainWindow::openFile()
 {
     QStringList fileNames = QFileDialog::getOpenFileNames(this, tr("Open File"),
-                            QString(),
-                            tr("All Supported Files (*.gpx *.kml *.pnt);;GPS Data (*.gpx);;Google Earth KML (*.kml);; Micro World Database II (*.pnt)"));
+                            m_lastFileOpenPath,
+                            tr("All Supported Files (*.gpx *.kml *.pnt *.osm);;GPS Data (*.gpx);;Google Earth KML (*.kml);;Micro World Database II (*.pnt);;OpenStreetMap Data (*.osm)"));
+
+    if ( !fileNames.isEmpty() ) {
+        const QString firstFile = fileNames.first();
+        m_lastFileOpenPath = QFileInfo( firstFile ).absolutePath();
+    }
 
     foreach( const QString &fileName, fileNames ) {
         m_controlView->marbleModel()->addGeoDataFile( fileName );
@@ -895,7 +900,7 @@ QString MainWindow::readMarbleDataPath()
      return marbleDataPath;
 }
 
-void MainWindow::readSettings()
+void MainWindow::readSettings(const QVariantMap& overrideSettings)
 {
      QSettings settings("kde.org", "Marble Desktop Globe");
 
@@ -916,12 +921,19 @@ void MainWindow::readSettings()
          showClouds(settings.value("showClouds", true ).toBool());
          workOffline(settings.value("workOffline", false ).toBool());
          showAtmosphere(settings.value("showAtmosphere", true ).toBool());
+         m_lastFileOpenPath = settings.value("lastFileOpenDir", QDir::homePath()).toString();
      settings.endGroup();
 
      setUpdatesEnabled(false);
 
      settings.beginGroup("MarbleWidget");
-         QString mapThemeId = settings.value("mapTheme", m_controlView->defaultMapThemeId() ).toString();
+         QString mapThemeId;
+         const QVariantMap::ConstIterator mapThemeIdIt = overrideSettings.find(QLatin1String("mapTheme"));
+         if ( mapThemeIdIt != overrideSettings.constEnd() ) {
+            mapThemeId = mapThemeIdIt.value().toString();
+         } else {
+            mapThemeId = settings.value("mapTheme", m_controlView->defaultMapThemeId() ).toString();
+         }
          qDebug() << "ReadSettings: mapThemeId: " << mapThemeId;
          m_controlView->marbleWidget()->setMapThemeId( mapThemeId );
          bool const smallScreen = MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen;
@@ -930,22 +942,40 @@ void MainWindow::readSettings()
             (Projection)(settings.value("projection", defaultProjection ).toInt())
          );
 
-         // Last location on quit
-         if ( m_configDialog->onStartup() == Marble::LastLocationVisited ) {
-            m_controlView->marbleWidget()->centerOn(
-                settings.value("quitLongitude", 0.0).toDouble(),
-                settings.value("quitLatitude", 0.0).toDouble() );
-            m_controlView->marbleWidget()->zoomView( settings.value("quitZoom", 1000).toInt() );
-         }
-
          // Set home position
          m_controlView->marbleModel()->setHome(
             settings.value("homeLongitude", 9.4).toDouble(),
             settings.value("homeLatitude", 54.8).toDouble(),
             settings.value("homeZoom", 1050 ).toInt()
          );
-         if ( m_configDialog->onStartup() == Marble::ShowHomeLocation ) {
-            m_controlView->marbleWidget()->goHome();
+
+         // Center on/Distance
+         const QVariantMap::ConstIterator distanceIt = overrideSettings.find(QLatin1String("distance"));
+         const bool isDistanceOverwritten = (distanceIt != overrideSettings.constEnd());
+
+         const QVariantMap::ConstIterator lonLatIt = overrideSettings.find(QLatin1String("lonlat"));
+         if ( lonLatIt != overrideSettings.constEnd() ) {
+            const QVariantList lonLat = lonLatIt.value().toList();
+            m_controlView->marbleWidget()->centerOn( lonLat.at(0).toDouble(), lonLat.at(1).toDouble() );
+         } else {
+            switch ( m_configDialog->onStartup() ) {
+            case Marble::LastLocationVisited:
+                m_controlView->marbleWidget()->centerOn(
+                    settings.value("quitLongitude", 0.0).toDouble(),
+                    settings.value("quitLatitude", 0.0).toDouble() );
+                if (! isDistanceOverwritten) {
+                    m_controlView->marbleWidget()->zoomView( settings.value("quitZoom", 1000).toInt() );
+                }
+                break;
+            case Marble::ShowHomeLocation:
+                m_controlView->marbleWidget()->goHome();
+                break;
+            default:
+                break;
+            }
+         }
+         if (isDistanceOverwritten) {
+             m_controlView->marbleWidget()->setDistance(distanceIt.value().toDouble());
          }
 
          bool isLocked = settings.value( "lockFloatItemPositions", false ).toBool();
@@ -954,16 +984,15 @@ void MainWindow::readSettings()
      settings.endGroup();
 
      settings.beginGroup( "Sun" );
-         m_controlView->sunLocator()->setShow( settings.value( "showSun", false ).toBool() );
-         m_controlView->sunLocator()->setCitylights( settings.value( "showCitylights", false ).toBool() );
-         m_controlView->sunLocator()->setCentered( settings.value( "centerOnSun", false ).toBool() );
+         m_controlView->marbleWidget()->setShowSunShading( settings.value( "showSun", false ).toBool() );
+         m_controlView->marbleWidget()->setShowCityLights( settings.value( "showCitylights", false ).toBool() );
+         m_controlView->marbleWidget()->setShowSunInZenith( settings.value( "centerOnSun", false ).toBool() );
      settings.endGroup();
 
      settings.beginGroup( "Time" );
         if( settings.value( "systemTime", "true" ).toBool() == true  )
         {
-            m_controlView->marbleModel()->setClockDateTime( QDateTime::currentDateTime().toUTC() );
-            m_controlView->marbleModel()->setClockSpeed( 1 );
+            /* nothing to do */
         }
         else if( settings.value( "lastSessionTime", "true" ).toBool() == true )
         {
@@ -976,9 +1005,14 @@ void MainWindow::readSettings()
 
      // Load previous route settings
      settings.beginGroup( "Routing" );
-     m_controlView->marbleModel()->routingManager()->readSettings();
-     bool const startupWarning = settings.value( "showGuidanceModeStartupWarning", QVariant( true ) ).toBool();
-     m_controlView->marbleModel()->routingManager()->setShowGuidanceModeStartupWarning( startupWarning );
+     {
+         RoutingManager *const routingManager = m_controlView->marbleModel()->routingManager();
+         routingManager->readSettings();
+         bool const startupWarning = settings.value( "showGuidanceModeStartupWarning", QVariant( true ) ).toBool();
+         routingManager->setShowGuidanceModeStartupWarning( startupWarning );
+         routingManager->setLastOpenPath( settings.value( "lastRouteOpenPath", QDir::homePath() ).toString() );
+         routingManager->setLastSavePath( settings.value( "lastRouteSavePath", QDir::homePath() ).toString() );
+     }
      settings.endGroup();
 
 
@@ -1019,11 +1053,12 @@ void MainWindow::readSettings()
     QString positionProvider = settings.value( "activePositionTrackingPlugin", QString() ).toString();
     if ( !positionProvider.isEmpty() ) {
         PositionTracking* tracking = m_controlView->marbleModel()->positionTracking();
-        PluginManager* pluginManager = m_controlView->marbleModel()->pluginManager();
+        const PluginManager* pluginManager = m_controlView->marbleModel()->pluginManager();
         QList<PositionProviderPlugin*> plugins = pluginManager->createPositionProviderPlugins();
         foreach( PositionProviderPlugin* plugin, plugins ) {
             if ( plugin->nameId() == positionProvider ) {
                 plugins.removeOne( plugin );
+                plugin->setMarbleModel( m_controlView->marbleModel() );
                 tracking->setPositionProviderPlugin( plugin );
                 break;
             }
@@ -1045,6 +1080,8 @@ void MainWindow::readSettings()
         trackingWidget->setRecenterMode( settings.value( "recenterMode", 0 ).toInt() );
         trackingWidget->setAutoZoom( settings.value( "autoZoom", false ).toBool() );
         trackingWidget->setTrackVisible( settings.value( "trackVisible", true ).toBool() );
+        trackingWidget->setLastOpenPath( settings.value( "lastTrackOpenPath", QDir::homePath() ).toString() );
+        trackingWidget->setLastSavePath( settings.value( "lastTrackSavePath", QDir::homePath() ).toString() );
     }
     settings.endGroup();
 
@@ -1070,6 +1107,7 @@ void MainWindow::writeSettings()
          settings.setValue( "showClouds", m_showCloudsAct->isChecked() );
          settings.setValue( "workOffline", m_workOfflineAct->isChecked() );
          settings.setValue( "showAtmosphere", m_showAtmosphereAct->isChecked() );
+         settings.setValue( "lastFileOpenDir", m_lastFileOpenPath );
      settings.endGroup();
 
      settings.beginGroup( "MarbleWidget" );
@@ -1101,9 +1139,9 @@ void MainWindow::writeSettings()
      settings.endGroup();
 
      settings.beginGroup( "Sun" );
-         settings.setValue( "showSun",        m_controlView->sunLocator()->getShow() );
-         settings.setValue( "showCitylights", m_controlView->sunLocator()->getCitylights() );
-         settings.setValue( "centerOnSun",    m_controlView->sunLocator()->getCentered() );
+         settings.setValue( "showSun",        m_controlView->marbleWidget()->showSunShading() );
+         settings.setValue( "showCitylights", m_controlView->marbleWidget()->showCityLights() );
+         settings.setValue( "centerOnSun",    m_controlView->marbleWidget()->showSunInZenith() );
      settings.endGroup();
 
       settings.beginGroup( "Time" );
@@ -1160,6 +1198,8 @@ void MainWindow::writeSettings()
          settings.setValue( "recenterMode", trackingWidget->recenterMode() );
          settings.setValue( "autoZoom", trackingWidget->autoZoom() );
          settings.setValue( "trackVisible", trackingWidget->trackVisible() );
+         settings.setValue( "lastTrackOpenPath", trackingWidget->lastOpenPath() );
+         settings.setValue( "lastTrackSavePath", trackingWidget->lastSavePath() );
      }
 
      settings.endGroup();
@@ -1169,9 +1209,13 @@ void MainWindow::writeSettings()
 
      // Store current route settings
      settings.beginGroup( "Routing" );
-     m_controlView->marbleModel()->routingManager()->writeSettings();
-     bool const startupWarning = m_controlView->marbleModel()->routingManager()->showGuidanceModeStartupWarning();
-     settings.setValue( "showGuidanceModeStartupWarning", startupWarning );
+     {
+         RoutingManager *const routingManager = m_controlView->marbleModel()->routingManager();
+         routingManager->writeSettings();
+         settings.setValue( "showGuidanceModeStartupWarning", routingManager->showGuidanceModeStartupWarning() );
+         settings.setValue( "lastRouteOpenPath", routingManager->lastOpenPath() );
+         settings.setValue( "lastRouteSavePath", routingManager->lastSavePath() );
+     }
      settings.endGroup();
 
      settings.beginGroup( "Navigation");
@@ -1198,7 +1242,7 @@ void MainWindow::updateSettings()
     m_controlView->marbleWidget()->setMapQualityForViewContext( m_configDialog->animationQuality(), Marble::Animation );
 
     m_controlView->marbleWidget()->setDefaultAngleUnit( m_configDialog->angleUnit() );
-    MarbleGlobal::getInstance()->locale()->setDistanceUnit( m_configDialog->distanceUnit() );
+    MarbleGlobal::getInstance()->locale()->setMeasurementSystem( m_configDialog->measurementSystem() );
 
     m_distance = m_controlView->marbleWidget()->distanceString();
     updateStatusBar();
@@ -1318,7 +1362,6 @@ void MainWindow::showRoutingDialog()
         m_routingWindow->setWindowTitle( tr( "Routing - Marble" ) );
 
         m_routingWidget = new RoutingWidget( m_controlView->marbleWidget(), m_routingWindow );
-        m_routingWidget->setWorkOffline( m_workOfflineAct->isChecked() );
         m_routingWidget->setShowDirectionsButtonVisible( true );
 
         QScrollArea* scrollArea = new QScrollArea;
@@ -1428,10 +1471,9 @@ void MainWindow::showGoToDialog()
     }
 
     m_gotoDialog->show();
-    m_gotoDialog->setWorkOffline( m_workOfflineAct->isChecked() );
     if ( m_gotoDialog->exec() == QDialog::Accepted ) {
-        GeoDataLookAt lookAt = m_gotoDialog->lookAt();
-        m_controlView->marbleWidget()->flyTo( lookAt );
+        const GeoDataCoordinates coordinates = m_gotoDialog->coordinates();
+        m_controlView->marbleWidget()->centerOn( coordinates );
     }
 }
 
