@@ -30,7 +30,6 @@
 
 #include "marble_export.h"
 #include "GeoDataCoordinates.h"       // In geodata/data/
-#include "Quaternion.h"
 
 // Qt
 class QAbstractItemModel;
@@ -49,6 +48,7 @@ class MarbleModel;
 class ViewportParams;
 class GeoPainter;
 class LayerInterface;
+class Quaternion;
 class RenderPlugin;
 class AbstractDataPlugin;
 class AbstractDataPluginItem;
@@ -125,6 +125,7 @@ class MARBLE_EXPORT MarbleMap : public QObject
 
     // Accessors to internal objects;
     ViewportParams *viewport();
+    const ViewportParams *viewport() const;
 
     void setMapQualityForViewContext( MapQuality qualityForViewContext, ViewContext viewContext );
     MapQuality mapQuality( ViewContext viewContext ) const;
@@ -208,17 +209,6 @@ class MARBLE_EXPORT MarbleMap : public QObject
 
     QVector<const GeoDataPlacemark*> whichFeatureAt( const QPoint& ) const;
 
-    /**
-     * @brief  Return the quaternion that specifies the rotation of the globe.
-     * @return The quaternion that describes the rotation of the globe.
-     */
-    Quaternion planetAxis() const;
-
-    /**
-     * @brief  Return a QPixmap with the current contents of the map.
-     */
-    QPixmap mapScreenShot();
-
     /**
      * @brief  Return the property value by name.
      * @return The property value (usually: visibility).
@@ -249,6 +239,24 @@ class MARBLE_EXPORT MarbleMap : public QObject
      */
     bool showClouds() const;
 
+    /**
+     * @brief  Return whether the night shadow is visible.
+     * @return visibility of night shadow
+     */
+    bool showSunShading() const;
+
+    /**
+     * @brief  Return whether the city lights are shown instead of the night shadow.
+     * @return visibility of city lights
+     */
+    bool showCityLights() const;
+
+    /**
+     * @brief  Return whether the sun is shown in the zenith.
+     * @return visibility of sun in the zenith
+     */
+    bool showSunInZenith() const;
+
     /**
      * @brief  Return whether the atmospheric glow is visible.
      * @return The cloud cover visibility.
@@ -297,12 +305,6 @@ class MARBLE_EXPORT MarbleMap : public QObject
      */
     bool showRelief() const;
 
-    /**
-     * @brief  Return whether the elevation model is visible.
-     * @return The elevation model visibility.
-     */
-    bool showElevationModel() const;
-
     /**
      * @brief  Return whether the ice layer is visible.
      * @return The ice layer visibility.
@@ -327,12 +329,6 @@ class MARBLE_EXPORT MarbleMap : public QObject
      */
     bool showLakes() const;
 
-    /**
-     * @brief Return whether Gps Data is visible.
-     * @return The Gps Data's visibility.
-     */
-    bool showGps() const;
-
     /**
      * @brief  Return whether the frame rate gets displayed.
      * @return the frame rates visibility
@@ -388,7 +384,7 @@ class MARBLE_EXPORT MarbleMap : public QObject
      * @param painter  The painter to use.
      * @param dirtyRect the rectangle that actually needs repainting.
      */
-    void paint( GeoPainter &painter, QRect &dirtyRect );
+    void paint( GeoPainter &painter, const QRect &dirtyRect );
 
     /**
      * @brief  Rotate the view by the two angles phi and theta.
@@ -403,12 +399,6 @@ class MARBLE_EXPORT MarbleMap : public QObject
      */
     void rotateBy( const qreal &deltaLon, const qreal &deltaLat );
 
-    /**
-     * @brief  Rotate the view by the angle specified by a Quaternion.
-     * @param  incRot a quaternion specifying the rotation
-     */
-    void rotateBy( const Quaternion& incRot );
-
     /**
      * @brief  Center the view on a geographical point
      * @param  lat  an angle parallel to the latitude lines
@@ -498,6 +488,24 @@ class MARBLE_EXPORT MarbleMap : public QObject
      */
     void setShowClouds( bool visible );
 
+    /**
+     * @brief  Set whether the night shadow is visible.
+     * @param  visibile visibility of shadow
+     */
+    void setShowSunShading( bool visible );
+
+    /**
+     * @brief  Set whether city lights instead of night shadow are visible.
+     * @param  visible visibility of city lights
+     */
+    void setShowCityLights( bool visible );
+
+    /**
+     * @brief Set whether the sun is visible in the zenith.
+     * @param visible  visibility of the sun in the zenith
+     */
+    void setShowSunInZenith( bool visible );
+
     /**
      * @brief Set whether the is tile is visible
      * NOTE: This is part of the transitional debug API
@@ -554,12 +562,6 @@ class MARBLE_EXPORT MarbleMap : public QObject
      */
     void setShowRelief( bool visible );
 
-    /**
-     * @brief  Set whether the elevation model is visible
-     * @param  visible  visibility of the elevation model
-     */
-    void setShowElevationModel( bool visible );
-
     /**
      * @brief  Set whether the ice layer is visible
      * @param  visible  visibility of the ice layer
@@ -584,12 +586,6 @@ class MARBLE_EXPORT MarbleMap : public QObject
      */
     void setShowLakes( bool visible );
 
-    /**
-     * @brief Set whether Gps Data is visible
-     * @param visible  visibility of the Gps Data
-     */
-    void setShowGps( bool visible );
-
     /**
      * @brief Set whether the frame rate gets shown
      * @param visible  visibility of the frame rate
@@ -672,13 +668,23 @@ class MARBLE_EXPORT MarbleMap : public QObject
     /**
      * @brief Enables custom drawing onto the MarbleMap straight after
      * @brief the globe and before all other layers have been rendered.
-     * @param painter 
+     * @param painter
+     *
+     * @deprecated implement LayerInterface and add it using @p addLayer()
      */
     virtual void customPaint( GeoPainter *painter );
 
+ private:
+    Q_PRIVATE_SLOT( d, void updateMapTheme() )
+    Q_PRIVATE_SLOT( d, void updateProperty( const QString &, bool ) )
+
  private:
     Q_DISABLE_COPY( MarbleMap )
     MarbleMapPrivate * const d;
+    friend class MarbleMapPrivate;
+
+    class CustomPaintLayer;
+    friend class CustomPaintLayer;
 
     TextureLayer *textureLayer();
 
