@@ -7,6 +7,7 @@
 //
 // Copyright 2006-2007 Torsten Rahn <tackat@kde.org>
 // Copyright 2007      Inge Wallin  <ingwa@kde.org>
+// Copyright 2010-2011 Bernhard Beschow <bbeschow@cs.tu-berlin.de>
 //
 
 #include "MarbleWidget.h"
@@ -16,7 +17,6 @@
 #include <QtCore/QHash>
 #include <QtCore/QSettings>
 #include <QtCore/QTime>
-#include <QtCore/QTimer>
 #include <QtGui/QItemSelectionModel>
 #include <QtGui/QPaintEvent>
 #include <QtGui/QRegion>
@@ -29,16 +29,20 @@
 
 #include "AbstractProjection.h"
 #include "DataMigration.h"
+#include "FpsLayer.h"
+#include "FileManager.h"
 #include "GeoDataLatLonAltBox.h"
+#include "GeoDataPlacemark.h"
 #include "GeoPainter.h"
+#include "MarbleClock.h"
 #include "MarbleDebug.h"
 #include "MarbleDirs.h"
 #include "MarbleLocale.h"
 #include "MarbleMap.h"
-#include "MarbleMap_p.h" // FIXME: remove this
 #include "MarbleModel.h"
 #include "MarblePhysics.h"
 #include "MarbleWidgetInputHandler.h"
+#include "MarbleWidgetPopupMenu.h"
 #include "MeasureTool.h"
 #include "Planet.h"
 #include "RenderPlugin.h"
@@ -52,34 +56,64 @@
 namespace Marble
 {
 
-const int REPAINT_SCHEDULING_INTERVAL = 1000;
+class MarbleWidget::CustomPaintLayer : public LayerInterface
+{
+ public:
+    CustomPaintLayer( MarbleWidget *widget )
+        : m_widget( widget )
+    {
+    }
+
+    virtual QStringList renderPosition() const { return QStringList() << "USER_TOOLS"; }
+
+    virtual bool render( GeoPainter *painter, ViewportParams *viewport,
+                         const QString &renderPos, GeoSceneLayer *layer )
+    {
+        Q_UNUSED( viewport );
+        Q_UNUSED( renderPos );
+        Q_UNUSED( layer );
+
+        m_widget->customPaint( painter );
+
+        return true;
+    }
+
+    virtual qreal zValue() const { return 1.0e7; }
+
+ private:
+    MarbleWidget *const m_widget;
+};
 
 
 class MarbleWidgetPrivate
 {
  public:
-    MarbleWidgetPrivate( MarbleMap *map, MarbleWidget *parent )
+    MarbleWidgetPrivate( MarbleWidget *parent )
         : m_widget( parent ),
-          m_map( map ),
-          m_model( map->model() ),
+          m_model(),
+          m_map( &m_model ),
           m_animationsEnabled( false ),
           m_logzoom( 0 ),
           m_zoomStep( MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen ? 60 : 40 ),
           m_inputhandler( 0 ),
-          m_physics( new MarblePhysics( parent ) ),
-          m_repaintTimer(),
+          m_physics( parent ),
           m_routingLayer( 0 ),
+          m_customPaintLayer( parent ),
+          m_popupmenu( 0 ),
+          m_showFrameRate( false ),
           m_viewAngle( 110.0 )
     {
     }
 
     ~MarbleWidgetPrivate()
     {
-        delete m_map;
+        m_map.removeLayer( &m_customPaintLayer );
     }
 
     void  construct();
 
+    void updateMapTheme();
+
     inline static qreal zoom( qreal radius ) { return (200.0 * log( radius ) ); }
     inline static qreal radius( qreal zoom ) { return pow( M_E, ( zoom / 200.0 ) ); }
 
@@ -99,10 +133,10 @@ class MarbleWidgetPrivate
       */
     void repaint();
 
-    MarbleWidget    *m_widget;
+    MarbleWidget    *const m_widget;
     // The model we are showing.
-    MarbleMap       *m_map;
-    MarbleModel     *m_model;   // Owned by m_map.  Don't delete.
+    MarbleModel     m_model;
+    MarbleMap       m_map;
 
     bool m_animationsEnabled;
 
@@ -112,12 +146,14 @@ class MarbleWidgetPrivate
 
     MarbleWidgetInputHandler  *m_inputhandler;
 
-    MarblePhysics    *m_physics;
-
-    // For scheduling repaints
-    QTimer           m_repaintTimer;
+    MarblePhysics    m_physics;
 
     RoutingLayer     *m_routingLayer;
+    MarbleWidget::CustomPaintLayer m_customPaintLayer;
+
+    MarbleWidgetPopupMenu *m_popupmenu;
+
+    bool             m_showFrameRate;
 
     const qreal      m_viewAngle;
 };
@@ -126,7 +162,7 @@ class MarbleWidgetPrivate
 
 MarbleWidget::MarbleWidget(QWidget *parent)
     : QWidget( parent ),
-      d( new MarbleWidgetPrivate( new MarbleMap(), this ) )
+      d( new MarbleWidgetPrivate( this ) )
 {
 //    setAttribute( Qt::WA_PaintOnScreen, true );
     d->construct();
@@ -158,76 +194,67 @@ void MarbleWidgetPrivate::construct()
     m_widget->setMinimumSize( 200, 300 );
     m_widget->setFocusPolicy( Qt::WheelFocus );
     m_widget->setFocus( Qt::OtherFocusReason );
-#if QT_VERSION >= 0x40600
-    m_widget->grabGesture(Qt::PinchGesture);
-#endif
-
-    // Initialize the map and forward some signals.
-    m_map->setSize( m_widget->width(), m_widget->height() );
-
-    m_widget->connect( m_map,    SIGNAL( projectionChanged( Projection ) ),
-                       m_widget, SIGNAL( projectionChanged( Projection ) ) );
-
-    // When some fundamental things change in the model, we got to
-    // show this in the view, i.e. here.
-    m_widget->connect( m_model,  SIGNAL( themeChanged( QString ) ),
-		       m_widget, SIGNAL( themeChanged( QString ) ) );
-    m_widget->connect( m_model, SIGNAL( modelChanged() ),
-                       m_widget, SLOT( update() ) );
-
-    // Repaint scheduling
-    m_widget->connect( m_map,    SIGNAL( repaintNeeded( QRegion ) ),
-                       m_widget, SLOT( scheduleRepaint( QRegion ) ) );
-    m_repaintTimer.setSingleShot( true );
-    m_repaintTimer.setInterval( REPAINT_SCHEDULING_INTERVAL );
-    m_widget->connect( &m_repaintTimer, SIGNAL( timeout() ),
-                       m_widget, SLOT( update() ) );
-
-    // When some fundamental things change in the map, we got to show
-    // this in the view, i.e. here.
-    m_widget->connect( m_map,    SIGNAL( tileLevelChanged( int ) ),
-                       m_widget, SIGNAL( tileLevelChanged( int ) ) );
-
-    m_widget->connect( m_map,    SIGNAL( pluginSettingsChanged() ),
-                       m_widget, SIGNAL( pluginSettingsChanged() ) );
-    m_widget->connect( m_map,    SIGNAL( renderPluginInitialized( RenderPlugin * ) ),
-                       m_widget, SIGNAL( renderPluginInitialized( RenderPlugin * ) ) );
 
     // Set background: black.
     m_widget->setPalette( QPalette ( Qt::black ) );
 
-    // Set whether the black space gets displayed or the earth gets simply 
+    // Set whether the black space gets displayed or the earth gets simply
     // displayed on the widget background.
     m_widget->setAutoFillBackground( true );
 
+    // Initialize the map and forward some signals.
+    m_map.setSize( m_widget->width(), m_widget->height() );
+    m_map.setShowFrameRate( false );  // never let the map draw the frame rate,
+                                       // we do this differently here in the widget
+
+    // forward some signals of m_map
+    m_widget->connect( &m_map,   SIGNAL( projectionChanged( Projection ) ),
+                       m_widget, SIGNAL( projectionChanged( Projection ) ) );
+    m_widget->connect( &m_map,   SIGNAL( tileLevelChanged( int ) ),
+                       m_widget, SIGNAL( tileLevelChanged( int ) ) );
+    m_widget->connect( &m_map,   SIGNAL( framesPerSecond( qreal ) ),
+                       m_widget, SIGNAL( framesPerSecond( qreal ) ) );
+
+    m_widget->connect( &m_map,   SIGNAL( pluginSettingsChanged() ),
+                       m_widget, SIGNAL( pluginSettingsChanged() ) );
+    m_widget->connect( &m_map,   SIGNAL( renderPluginInitialized( RenderPlugin * ) ),
+                       m_widget, SIGNAL( renderPluginInitialized( RenderPlugin * ) ) );
+
+    // react to some signals of m_map
+    m_widget->connect( &m_map,   SIGNAL( themeChanged( QString ) ),
+                       m_widget, SLOT( updateMapTheme() ) );
+    m_widget->connect( &m_map,   SIGNAL( repaintNeeded( QRegion ) ),
+                       m_widget, SLOT( update() ) );
+
+    m_widget->connect( m_model.fileManager(), SIGNAL( centeredDocument(GeoDataLatLonBox) ),
+                       m_widget, SLOT( centerOn(GeoDataLatLonBox) ) );
+
+
     // Show a progress dialog when the model calculates new map tiles.
-    m_widget->connect( m_model, SIGNAL( creatingTilesStart( TileCreator*, const QString&,
-                                                            const QString& ) ),
+    m_widget->connect( &m_model, SIGNAL( creatingTilesStart( TileCreator*, const QString&,
+                                                             const QString& ) ),
                        m_widget, SLOT( creatingTilesStart( TileCreator*, const QString&,
                                                            const QString& ) ) );
 
-    m_widget->connect( m_model->sunLocator(), SIGNAL( enableWidgetInput( bool ) ),
-                       m_widget, SLOT( setInputEnabled( bool ) ) );
+    m_popupmenu = new MarbleWidgetPopupMenu( m_widget, &m_model );
 
-    m_widget->connect( m_model->sunLocator(), SIGNAL( updateStars() ),
-                       m_widget, SLOT( update() ) );
-
-    m_widget->connect( m_model->sunLocator(), SIGNAL( centerSun( qreal, qreal ) ),
-                       m_widget, SLOT( centerOn( qreal, qreal ) ) );
+    m_widget->connect( m_popupmenu, SIGNAL( trackPlacemark( const GeoDataPlacemark* ) ),
+                       &m_model, SLOT( setTrackedPlacemark( const GeoDataPlacemark* ) ) );
 
     m_widget->setInputHandler( new MarbleWidgetDefaultInputHandler( m_widget ) );
-    m_widget->setMouseTracking( m_widget );
+    m_widget->setMouseTracking( true );
 
     m_routingLayer = new RoutingLayer( m_widget, m_widget );
-    m_routingLayer->setRouteRequest( m_model->routingManager()->routeRequest() );
-    m_routingLayer->setModel( m_model->routingManager()->routingModel() );
-    m_map->addLayer( m_routingLayer );
+    m_routingLayer->setRouteRequest( m_model.routingManager()->routeRequest() );
+    m_routingLayer->setPlacemarkModel( 0 );
 
     m_widget->connect( m_routingLayer, SIGNAL( routeDirty() ),
-                       m_model->routingManager(), SLOT( updateRoute() ) );
-    m_widget->connect( m_model->routingManager()->alternativeRoutesModel(),
+                       m_model.routingManager(), SLOT( updateRoute() ) );
+    m_widget->connect( m_model.routingManager()->alternativeRoutesModel(),
                        SIGNAL( currentRouteChanged( GeoDataDocument* ) ),
                        m_widget, SLOT( repaint() ) );
+
+    m_map.addLayer( &m_customPaintLayer );
 }
 
 void MarbleWidgetPrivate::moveByStep( int stepsRight, int stepsDown, FlyToMode mode )
@@ -243,9 +270,9 @@ void MarbleWidgetPrivate::repaint()
     // We only have to repaint the background every time if the earth
     // doesn't cover the whole image.
     m_widget->setAttribute( Qt::WA_NoSystemBackground,
-                  m_widget->viewport()->mapCoversViewport() && !m_model->mapThemeId().isEmpty() );
+                  m_widget->viewport()->mapCoversViewport() && !m_model.mapThemeId().isEmpty() );
 
-    m_widget->repaint();
+    m_widget->update();
 }
 
 // ----------------------------------------------------------------
@@ -253,18 +280,23 @@ void MarbleWidgetPrivate::repaint()
 
 MarbleModel *MarbleWidget::model() const
 {
-    return d->m_model;
+    return &d->m_model;
 }
 
 
 ViewportParams* MarbleWidget::viewport()
 {
-    return d->m_map->viewport();
+    return d->m_map.viewport();
 }
 
 const ViewportParams* MarbleWidget::viewport() const
 {
-    return d->m_map->viewport();
+    return d->m_map.viewport();
+}
+
+MarbleWidgetPopupMenu *MarbleWidget::popupMenu()
+{
+    return d->m_popupmenu;
 }
 
 
@@ -289,44 +321,40 @@ MarbleWidgetInputHandler *MarbleWidget::inputHandler() const
   return d->m_inputhandler;
 }
 
-Quaternion MarbleWidget::planetAxis() const
-{
-    return viewport()->planetAxis();
-}
-
-
 int MarbleWidget::radius() const
 {
-    return d->m_map->radius();
+    return d->m_map.radius();
 }
 
 void MarbleWidget::setRadius( int radius )
 {
     Q_ASSERT( radius >= 0 );
-    if ( radius == d->m_map->radius() ) {
-        return;
-    }
+    bool adjustRadius = radius != d->m_map.radius();
 
     qreal const zoom = d->zoom( radius );
 
     // Prevent exceeding zoom range
     if ( zoom < minimumZoom() ) {
         radius = d->radius( minimumZoom() );
+        adjustRadius = true;
     } else if ( zoom > maximumZoom() ) {
         radius = d->radius( maximumZoom() );
+        adjustRadius = true;
     }
 
-    d->m_map->setRadius( radius );
-    d->m_logzoom = qRound( zoom );
+    if( adjustRadius) {
+        d->m_map.setRadius( radius );
+        d->m_logzoom = qRound( zoom );
 
-    emit zoomChanged( d->m_logzoom );
-    emit distanceChanged( distanceString() );
-    emit visibleLatLonAltBoxChanged( d->m_map->viewport()->viewLatLonAltBox() );
+        emit zoomChanged( d->m_logzoom );
+        emit distanceChanged( distanceString() );
+        emit visibleLatLonAltBoxChanged( d->m_map.viewport()->viewLatLonAltBox() );
 
-    d->repaint();
+        d->repaint();
+    }
 }
 
-qreal MarbleWidget::moveStep()
+qreal MarbleWidget::moveStep() const
 {
     if ( radius() < qSqrt( (qreal)(width() * width() + height() * height()) ) )
         return 180.0 * 0.1;
@@ -342,52 +370,52 @@ int MarbleWidget::zoom() const
 
 int MarbleWidget::tileZoomLevel() const
 {
-    return d->m_map->tileZoomLevel();
+    return d->m_map.tileZoomLevel();
 }
 
 int  MarbleWidget::minimumZoom() const
 {
-    return d->m_map->minimumZoom();
+    return d->m_map.minimumZoom();
 }
 
 int  MarbleWidget::maximumZoom() const
 {
-    return d->m_map->maximumZoom();
+    return d->m_map.maximumZoom();
 }
 
 QVector<const GeoDataPlacemark*> MarbleWidget::whichFeatureAt( const QPoint &curpos ) const
 {
-    return d->m_map->whichFeatureAt( curpos );
+    return d->m_map.whichFeatureAt( curpos );
 }
 
 QList<AbstractDataPluginItem*> MarbleWidget::whichItemAt( const QPoint &curpos ) const
 {
-    return d->m_map->whichItemAt( curpos );
+    return d->m_map.whichItemAt( curpos );
 }
 
 void MarbleWidget::addLayer( LayerInterface *layer )
 {
-    d->m_map->addLayer( layer );
+    d->m_map.addLayer( layer );
 }
 
 void MarbleWidget::removeLayer( LayerInterface *layer )
 {
-    d->m_map->removeLayer( layer );
+    d->m_map.removeLayer( layer );
 }
 
 TextureLayer *MarbleWidget::textureLayer()
 {
-    return d->m_map->textureLayer();
+    return d->m_map.textureLayer();
 }
 
 const Marble::TextureLayer* MarbleWidget::textureLayer() const
 {
-    return d->m_map->textureLayer();
+    return d->m_map.textureLayer();
 }
 
 MeasureTool *MarbleWidget::measureTool()
 {
-    return d->m_map->measureTool();
+    return d->m_map.measureTool();
 }
 
 QPixmap MarbleWidget::mapScreenShot()
@@ -397,107 +425,112 @@ QPixmap MarbleWidget::mapScreenShot()
 
 bool MarbleWidget::showOverviewMap() const
 {
-    return d->m_map->showOverviewMap();
+    return d->m_map.showOverviewMap();
 }
 
 bool MarbleWidget::showScaleBar() const
 {
-    return d->m_map->showScaleBar();
+    return d->m_map.showScaleBar();
 }
 
 bool MarbleWidget::showCompass() const
 {
-    return d->m_map->showCompass();
+    return d->m_map.showCompass();
 }
 
 bool MarbleWidget::showClouds() const
 {
-    return d->m_map->showClouds();
+    return d->m_map.showClouds();
+}
+
+bool MarbleWidget::showSunShading() const
+{
+    return d->m_map.showSunShading();
+}
+
+bool MarbleWidget::showCityLights() const
+{
+    return d->m_map.showCityLights();
+}
+
+bool MarbleWidget::showSunInZenith() const
+{
+    return d->m_map.showSunInZenith();
 }
 
 bool MarbleWidget::showAtmosphere() const
 {
-    return d->m_map->showAtmosphere();
+    return d->m_map.showAtmosphere();
 }
 
 bool MarbleWidget::showCrosshairs() const
 {
-    return d->m_map->showCrosshairs();
+    return d->m_map.showCrosshairs();
 }
 
 bool MarbleWidget::showGrid() const
 {
-    return d->m_map->showGrid();
+    return d->m_map.showGrid();
 }
 
 bool MarbleWidget::showPlaces() const
 {
-    return d->m_map->showPlaces();
+    return d->m_map.showPlaces();
 }
 
 bool MarbleWidget::showCities() const
 {
-    return d->m_map->showCities();
+    return d->m_map.showCities();
 }
 
 bool MarbleWidget::showTerrain() const
 {
-    return d->m_map->showTerrain();
+    return d->m_map.showTerrain();
 }
 
 bool MarbleWidget::showOtherPlaces() const
 {
-    return d->m_map->showOtherPlaces();
+    return d->m_map.showOtherPlaces();
 }
 
 bool MarbleWidget::showRelief() const
 {
-    return d->m_map->showRelief();
-}
-
-bool MarbleWidget::showElevationModel() const
-{
-    return d->m_map->showElevationModel();
+    return d->m_map.showRelief();
 }
 
 bool MarbleWidget::showIceLayer() const
 {
-    return d->m_map->showIceLayer();
+    return d->m_map.showIceLayer();
 }
 
 bool MarbleWidget::showBorders() const
 {
-    return d->m_map->showBorders();
+    return d->m_map.showBorders();
 }
 
 bool MarbleWidget::showRivers() const
 {
-    return d->m_map->showRivers();
+    return d->m_map.showRivers();
 }
 
 bool MarbleWidget::showLakes() const
 {
-    return d->m_map->showLakes();
-}
-
-bool MarbleWidget::showGps() const
-{
-    return d->m_map->showGps();
+    return d->m_map.showLakes();
 }
 
 bool MarbleWidget::showFrameRate() const
 {
-    return d->m_map->showFrameRate();
+    return d->m_showFrameRate;
 }
 
 bool MarbleWidget::showBackground() const
 {
-    return d->m_map->showBackground();
+    return d->m_map.showBackground();
 }
 
 quint64 MarbleWidget::volatileTileCacheLimit() const
 {
-    return d->m_map->volatileTileCacheLimit();
+    return d->m_map.volatileTileCacheLimit();
 }
 
 
@@ -515,12 +548,12 @@ void MarbleWidget::zoomView( int newZoom, FlyToMode mode )
         if ( newZoom  == d->m_logzoom )
             return;
 
-        d->m_map->setRadius( d->radius( newZoom ) );
+        d->m_map.setRadius( d->radius( newZoom ) );
         d->m_logzoom = newZoom;
 
         emit zoomChanged( d->m_logzoom );
         emit distanceChanged( distanceString() );
-        emit visibleLatLonAltBoxChanged( d->m_map->viewport()->viewLatLonAltBox() );
+        emit visibleLatLonAltBoxChanged( d->m_map.viewport()->viewLatLonAltBox() );
 
         d->repaint();
     }
@@ -541,10 +574,10 @@ void MarbleWidget::zoomViewBy( int zoomStep, FlyToMode mode )
 
 void MarbleWidget::zoomIn( FlyToMode mode )
 {
-    if ( d->m_map->tileZoomLevel() < 0 ) {
+    if ( d->m_map.tileZoomLevel() < 0 ) {
         zoomViewBy( d->m_zoomStep, mode );
     } else {
-        int radius = d->m_map->preferredRadiusCeil( d->m_map->radius() * 1.05 ) + 2;
+        int radius = d->m_map.preferredRadiusCeil( d->m_map.radius() * 1.05 ) + 2;
         radius = qMax<int>( d->radius( minimumZoom() ), qMin<int>( radius, d->radius( maximumZoom() ) ) );
 
         GeoDataLookAt target = lookAt();
@@ -556,10 +589,10 @@ void MarbleWidget::zoomIn( FlyToMode mode )
 
 void MarbleWidget::zoomOut( FlyToMode mode )
 {
-    if ( d->m_map->tileZoomLevel() < 0 ) {
+    if ( d->m_map.tileZoomLevel() < 0 ) {
         zoomViewBy( -d->m_zoomStep, mode );
     } else {
-        int radius = d->m_map->preferredRadiusFloor( d->m_map->radius() * 0.95 ) + 2;
+        int radius = d->m_map.preferredRadiusFloor( d->m_map.radius() * 0.95 ) + 2;
         radius = qMax<int>( d->radius( minimumZoom() ), qMin<int>( radius, d->radius( maximumZoom() ) ) );
 
         GeoDataLookAt target = lookAt();
@@ -569,18 +602,12 @@ void MarbleWidget::zoomOut( FlyToMode mode )
     }
 }
 
-void MarbleWidget::rotateBy( const Quaternion& incRot )
-{
-    d->m_map->rotateBy( incRot );
-    d->repaint();
-}
-
 void MarbleWidget::rotateBy( const qreal deltaLon, const qreal deltaLat, FlyToMode mode )
 {
     Quaternion  rotPhi( 1.0, deltaLat / 180.0, 0.0, 0.0 );
     Quaternion  rotTheta( 1.0, 0.0, deltaLon / 180.0, 0.0 );
 
-    Quaternion  axis = planetAxis();
+    Quaternion  axis = d->m_map.viewport()->planetAxis();
     qreal lon( 0.0 ), lat( 0.0 );
     axis.getSpherical( lon, lat );
     axis = rotTheta * axis;
@@ -605,30 +632,45 @@ void MarbleWidget::centerOn( const qreal lon, const qreal lat, bool animated )
 void MarbleWidget::centerOn( const GeoDataCoordinates &position, bool animated )
 {
     GeoDataLookAt target = lookAt();
-    target.setLongitude( position.longitude() );
-    target.setLatitude( position.latitude() );
+    target.setCoordinates( position );
     flyTo( target, animated ? Automatic : Instant );
 }
 
 void MarbleWidget::centerOn( const GeoDataLatLonBox &box, bool animated )
 {
+    int newRadius = radius();
     ViewportParams* viewparams = viewport();
     //prevent divide by zero
     if( box.height() && box.width() ) {
         //work out the needed zoom level
         int const horizontalRadius = ( 0.25 * M_PI ) * ( viewparams->height() / box.height() );
         int const verticalRadius = ( 0.25 * M_PI ) * ( viewparams->width() / box.width() );
-        int const radius = qMin<int>( horizontalRadius, verticalRadius );
-        // radius < 0 can happen if the box size approaches zero
-        setRadius( radius < 0 ? d->radius( maximumZoom() ) : radius );
+        newRadius = qMin<int>( horizontalRadius, verticalRadius );
+        newRadius = qMax<int>( d->radius( minimumZoom() ), qMin<int>( newRadius, d->radius( maximumZoom() ) ) );
     }
 
     //move the map
-    centerOn( box.center().longitude( GeoDataCoordinates::Degree ),
-              box.center().latitude( GeoDataCoordinates::Degree ),
-              animated );
+    GeoDataLookAt target;
+    target.setCoordinates( box.center() );
+    target.setAltitude( box.center().altitude() );
+    target.setRange(KM2METER * distanceFromRadius( newRadius ));
+    flyTo( target, animated ? Automatic : Instant );
+}
 
-    repaint();
+void MarbleWidget::centerOn( const GeoDataPlacemark& placemark, bool animated )
+{
+    GeoDataLookAt *lookAt( placemark.lookAt() );
+    if ( lookAt ) {
+        flyTo( *lookAt, animated ? Automatic : Instant );
+    } else {
+        bool icon;
+        GeoDataCoordinates coords = placemark.coordinate( d->m_model.clock()->dateTime(), &icon );
+        if ( icon ) {
+            centerOn( coords, animated );
+        } else {
+            centerOn( placemark.geometry()->latLonAltBox(), animated );
+        }
+    }
 }
 
 void MarbleWidget::setCenterLatitude( qreal lat, FlyToMode mode )
@@ -643,12 +685,12 @@ void MarbleWidget::setCenterLongitude( qreal lon, FlyToMode mode )
 
 Projection MarbleWidget::projection() const
 {
-    return d->m_map->projection();
+    return d->m_map.projection();
 }
 
 void MarbleWidget::setProjection( Projection projection )
 {
-    d->m_map->setProjection( projection );
+    d->m_map.setProjection( projection );
     d->repaint();
 }
 
@@ -686,7 +728,7 @@ void MarbleWidget::leaveEvent( QEvent* )
 void MarbleWidget::resizeEvent( QResizeEvent* )
 {
     setUpdatesEnabled( false );
-    d->m_map->setSize( width(), height() );
+    d->m_map.setSize( width(), height() );
     d->repaint();
     setUpdatesEnabled( true );
 }
@@ -708,42 +750,35 @@ void MarbleWidget::disconnectNotify( const char * signal )
 }
 
 bool MarbleWidget::screenCoordinates( qreal lon, qreal lat,
-                                      qreal& x, qreal& y )
+                                      qreal& x, qreal& y ) const
 {
-    return d->m_map->screenCoordinates( lon, lat, x, y );
+    return d->m_map.screenCoordinates( lon, lat, x, y );
 }
 
 bool MarbleWidget::geoCoordinates( int x, int y,
                                    qreal& lon, qreal& lat,
-                                   GeoDataCoordinates::Unit unit )
+                                   GeoDataCoordinates::Unit unit ) const
 {
-    return d->m_map->geoCoordinates( x, y, lon, lat, unit );
+    return d->m_map.geoCoordinates( x, y, lon, lat, unit );
 }
 
 qreal MarbleWidget::centerLatitude() const
 {
-    return d->m_map->centerLatitude();
+    return d->m_map.centerLatitude();
 }
 
 qreal MarbleWidget::centerLongitude() const
 {
-    return d->m_map->centerLongitude();
+    return d->m_map.centerLongitude();
 }
 
-QRegion MarbleWidget::activeRegion()
-{
-    return viewport()->activeRegion();
-}
-
-QRegion MarbleWidget::mapRegion()
+QRegion MarbleWidget::mapRegion() const
 {
     return viewport()->currentProjection()->mapRegion( viewport() );
 }
 
 void MarbleWidget::paintEvent( QPaintEvent *evt )
 {
-    // Stop repaint timer if it is already running
-    d->m_repaintTimer.stop();
     QTime t;
     t.start();
 
@@ -759,7 +794,7 @@ void MarbleWidget::paintEvent( QPaintEvent *evt )
     {
         // If the globe covers fully the screen then we can use the faster
         // RGB32 as there are no translucent areas involved.
-        QImage::Format imageFormat = ( d->m_map->mapCoversViewport() )
+        QImage::Format imageFormat = ( d->m_map.mapCoversViewport() )
                                      ? QImage::Format_RGB32
                                      : QImage::Format_ARGB32_Premultiplied;
         // Paint to an intermediate image
@@ -769,15 +804,11 @@ void MarbleWidget::paintEvent( QPaintEvent *evt )
     }
 
     // Create a painter that will do the painting.
-    GeoPainter painter( paintDevice, d->m_map->viewport(),
-                        d->m_map->mapQuality(), doClip );
+    GeoPainter painter( paintDevice, d->m_map.viewport(),
+                        d->m_map.mapQuality(), doClip );
     QRect  dirtyRect = evt->rect();
 
-    // Draws the map like MarbleMap::paint does, but adds our customPaint in between
-    d->m_map->d->paintGround( painter, dirtyRect );
-    d->m_map->customPaint( &painter );
-    customPaint( &painter );
-    d->m_map->measureTool()->render( &painter, viewport() );
+    d->m_map.paint( painter, dirtyRect );
 
     if ( !isEnabled() )
     {
@@ -788,16 +819,18 @@ void MarbleWidget::paintEvent( QPaintEvent *evt )
             *pixel = qRgb( gray, gray, gray );
         }
 
-        GeoPainter widgetPainter( this, d->m_map->viewport(),
-                            d->m_map->mapQuality(), doClip );
+        GeoPainter widgetPainter( this, d->m_map.viewport(),
+                            d->m_map.mapQuality(), doClip );
         widgetPainter.drawImage( rect(), image );
     }
 
-    if ( showFrameRate() )
+    if ( d->m_showFrameRate )
     {
-        qreal fps = 1000.0 / (qreal)( t.elapsed() + 1 );
-        d->m_map->d->paintFps( painter, dirtyRect, fps );
-        emit d->m_map->framesPerSecond( fps );
+        FpsLayer fpsLayer( &t );
+        fpsLayer.render( &painter, d->m_map.viewport() );
+
+        const qreal fps = 1000.0 / (qreal)( t.elapsed() + 1 );
+        emit framesPerSecond( fps );
     }
 }
 
@@ -813,7 +846,7 @@ void MarbleWidget::goHome( FlyToMode mode )
     qreal  homeLon = 0;
     qreal  homeLat = 0;
     int homeZoom = 0;
-    d->m_model->home( homeLon, homeLat, homeZoom );
+    d->m_model.home( homeLon, homeLat, homeZoom );
 
     GeoDataLookAt target;
     target.setLongitude( homeLon, GeoDataCoordinates::Degree );
@@ -825,189 +858,204 @@ void MarbleWidget::goHome( FlyToMode mode )
 
 QString MarbleWidget::mapThemeId() const
 {
-    return d->m_model->mapThemeId();
+    return d->m_model.mapThemeId();
 }
 
 void MarbleWidget::setMapThemeId( const QString& mapThemeId )
 {
-    if ( !mapThemeId.isEmpty() && mapThemeId == d->m_model->mapThemeId() )
-        return;
-    
-    d->m_map->setMapThemeId( mapThemeId );
+    d->m_map.setMapThemeId( mapThemeId );
+}
 
-    // Now we want a full repaint as the atmosphere might differ
-    setAttribute( Qt::WA_NoSystemBackground,
-                  false );
+void MarbleWidgetPrivate::updateMapTheme()
+{
+    m_map.removeLayer( m_routingLayer );
 
-    SunLocator  *sunLocator = d->m_model->sunLocator();
+    m_widget->setRadius( m_widget->radius() ); // Corrects zoom range, if needed
 
-    if ( sunLocator && sunLocator->getCentered() ) {
-        qreal  lon = sunLocator->getLon();
-        qreal  lat = sunLocator->getLat();
-        centerOn( lon, lat );
-
-        setInputEnabled( false );
+    if ( m_model.planetId() == "earth" ) {
+        m_map.addLayer( m_routingLayer );
     }
 
-    repaint();
+    emit m_widget->themeChanged( m_map.mapThemeId() );
+
+    // Now we want a full repaint as the atmosphere might differ
+    m_widget->setAttribute( Qt::WA_NoSystemBackground, false );
+
+    m_widget->update();
 }
 
 GeoSceneDocument *MarbleWidget::mapTheme() const
 {
-    return d->m_model->mapTheme();
+    return d->m_model.mapTheme();
 }
 
 void MarbleWidget::setPropertyValue( const QString& name, bool value )
 {
     mDebug() << "In MarbleWidget the property " << name << "was set to " << value;
-    d->m_map->setPropertyValue( name, value );
+    d->m_map.setPropertyValue( name, value );
 
-    repaint();
+    update();
 }
 
 void MarbleWidget::setShowOverviewMap( bool visible )
 {
-    d->m_map->setShowOverviewMap( visible );
+    d->m_map.setShowOverviewMap( visible );
 
-    repaint();
+    update();
 }
 
 void MarbleWidget::setShowScaleBar( bool visible )
 {
-    d->m_map->setShowScaleBar( visible );
+    d->m_map.setShowScaleBar( visible );
 
-    repaint();
+    update();
 }
 
 void MarbleWidget::setShowCompass( bool visible )
 {
-    d->m_map->setShowCompass( visible );
+    d->m_map.setShowCompass( visible );
 
-    repaint();
+    update();
 }
 
 void MarbleWidget::setShowClouds( bool visible )
 {
-    d->m_map->setShowClouds( visible );
+    d->m_map.setShowClouds( visible );
 
-    repaint();
+    update();
+}
+
+void MarbleWidget::setShowSunShading( bool visible )
+{
+    d->m_map.setShowSunShading( visible );
+
+    update();
+}
+
+void MarbleWidget::setShowCityLights( bool visible )
+{
+    d->m_map.setShowCityLights( visible );
+
+    update();
+}
+
+void MarbleWidget::setShowSunInZenith( bool visible )
+{
+    disconnect( d->m_model.sunLocator(), SIGNAL( positionChanged( qreal, qreal ) ),
+                this,                    SLOT( centerOn( qreal, qreal ) ) );
+
+    if ( d->m_map.showSunInZenith() != visible ) { // Toggling input modifies event filters, so avoid that if not needed
+        d->m_map.setShowSunInZenith( visible );
+        setInputEnabled( !d->m_map.showSunInZenith() );
+    }
+
+    if ( d->m_map.showSunInZenith() ) {
+        connect( d->m_model.sunLocator(), SIGNAL( positionChanged( qreal, qreal ) ),
+                 this,                    SLOT( centerOn( qreal, qreal ) ) );
+    }
 }
 
 void MarbleWidget::setShowAtmosphere( bool visible )
 {
-    d->m_map->setShowAtmosphere( visible );
+    d->m_map.setShowAtmosphere( visible );
 
-    repaint();
+    update();
 }
 
 void MarbleWidget::setShowCrosshairs( bool visible )
 {
-    d->m_map->setShowCrosshairs( visible );
+    d->m_map.setShowCrosshairs( visible );
 
-    repaint();
+    update();
 }
 
 void MarbleWidget::setShowGrid( bool visible )
 {
-    d->m_map->setShowGrid( visible );
+    d->m_map.setShowGrid( visible );
 
-    repaint();
+    update();
 }
 
 void MarbleWidget::setShowPlaces( bool visible )
 {
-    d->m_map->setShowPlaces( visible );
+    d->m_map.setShowPlaces( visible );
 
-    repaint();
+    update();
 }
 
 void MarbleWidget::setShowCities( bool visible )
 {
-    d->m_map->setShowCities( visible );
+    d->m_map.setShowCities( visible );
 
-    repaint();
+    update();
 }
 
 void MarbleWidget::setShowTerrain( bool visible )
 {
-    d->m_map->setShowTerrain( visible );
+    d->m_map.setShowTerrain( visible );
 
-    repaint();
+    update();
 }
 
 void MarbleWidget::setShowOtherPlaces( bool visible )
 {
-    d->m_map->setShowOtherPlaces( visible );
+    d->m_map.setShowOtherPlaces( visible );
 
-    repaint();
+    update();
 }
 
 void MarbleWidget::setShowRelief( bool visible )
 {
-    d->m_map->setShowRelief( visible );
+    d->m_map.setShowRelief( visible );
 
-    repaint();
-}
-
-void MarbleWidget::setShowElevationModel( bool visible )
-{
-    d->m_map->setShowElevationModel( visible );
-
-    repaint();
+    update();
 }
 
 void MarbleWidget::setShowIceLayer( bool visible )
 {
-    d->m_map->setShowIceLayer( visible );
+    d->m_map.setShowIceLayer( visible );
 
-    repaint();
+    update();
 }
 
 void MarbleWidget::setShowBorders( bool visible )
 {
-    d->m_map->setShowBorders( visible );
+    d->m_map.setShowBorders( visible );
 
-    repaint();
+    update();
 }
 
 void MarbleWidget::setShowRivers( bool visible )
 {
-    d->m_map->setShowRivers( visible );
+    d->m_map.setShowRivers( visible );
 
-    repaint();
+    update();
 }
 
 void MarbleWidget::setShowLakes( bool visible )
 {
-    d->m_map->setShowLakes( visible );
+    d->m_map.setShowLakes( visible );
 
-    repaint();
+    update();
 }
 
 void MarbleWidget::setShowFrameRate( bool visible )
 {
-    d->m_map->setShowFrameRate( visible );
+    d->m_showFrameRate = visible;
 
-    repaint();
+    update();
 }
 
 void MarbleWidget::setShowBackground( bool visible )
 {
-    d->m_map->setShowBackground( visible );
+    d->m_map.setShowBackground( visible );
 
-    repaint();
-}
-
-void MarbleWidget::setShowGps( bool visible )
-{
-    d->m_map->setShowGps( visible );
-
-    repaint();
+    update();
 }
 
 void MarbleWidget::setShowTileId( bool visible )
 {
-    d->m_map->setShowTileId( visible );
+    d->m_map.setShowTileId( visible );
 }
 
 void MarbleWidget::notifyMouseClick( int x, int y)
@@ -1025,12 +1073,12 @@ void MarbleWidget::notifyMouseClick( int x, int y)
 void MarbleWidget::clearVolatileTileCache()
 {
     mDebug() << "About to clear VolatileTileCache";
-    d->m_map->clearVolatileTileCache();
+    d->m_map.clearVolatileTileCache();
 }
 
 void MarbleWidget::setVolatileTileCacheLimit( quint64 kiloBytes )
 {
-    d->m_map->setVolatileTileCacheLimit( kiloBytes );
+    d->m_map.setVolatileTileCacheLimit( kiloBytes );
 }
 
 // This slot will called when the Globe starts to create the tiles.
@@ -1044,42 +1092,36 @@ void MarbleWidget::creatingTilesStart( TileCreator *creator,
     dlg.exec();
 }
 
-void MarbleWidget::scheduleRepaint( const QRegion& dirtyRegion )
+MapQuality MarbleWidget::mapQuality( ViewContext viewContext ) const
 {
-    Q_UNUSED( dirtyRegion );
-    if ( !d->m_repaintTimer.isActive() ) {
-        d->m_repaintTimer.start();
-    }
-}
-
-MapQuality MarbleWidget::mapQuality( ViewContext viewContext )
-{
-    return d->m_map->mapQuality( viewContext );
+    return d->m_map.mapQuality( viewContext );
 }
 
 void MarbleWidget::setMapQualityForViewContext( MapQuality quality, ViewContext viewContext )
 {
-    const MapQuality oldQuality = d->m_map->mapQuality();
+    const MapQuality oldQuality = d->m_map.mapQuality();
 
-    d->m_map->setMapQualityForViewContext( quality, viewContext );
+    d->m_map.setMapQualityForViewContext( quality, viewContext );
 
-    if ( d->m_map->mapQuality() != oldQuality )
+    if ( d->m_map.mapQuality() != oldQuality )
         d->repaint();
 }
 
 ViewContext MarbleWidget::viewContext() const
 {
-    return d->m_map->viewContext();
+    return d->m_map.viewContext();
 }
 
 void MarbleWidget::setViewContext( ViewContext viewContext )
 {
-    const MapQuality oldQuality = d->m_map->mapQuality();
+    if ( d->m_map.viewContext() != viewContext ) {
+        const MapQuality oldQuality = d->m_map.mapQuality();
+        d->m_map.setViewContext( viewContext );
+        d->m_routingLayer->setViewContext( viewContext );
 
-    d->m_map->setViewContext( viewContext );
-
-    if ( d->m_map->mapQuality() != oldQuality )
-        d->repaint();
+        if ( d->m_map.mapQuality() != oldQuality )
+            d->repaint();
+    }
 }
 
 bool MarbleWidget::animationsEnabled() const
@@ -1094,22 +1136,22 @@ void MarbleWidget::setAnimationsEnabled( bool enabled )
 
 AngleUnit MarbleWidget::defaultAngleUnit() const
 {
-    return d->m_map->defaultAngleUnit();
+    return d->m_map.defaultAngleUnit();
 }
 
 void MarbleWidget::setDefaultAngleUnit( AngleUnit angleUnit )
 {
-    d->m_map->setDefaultAngleUnit( angleUnit );
+    d->m_map.setDefaultAngleUnit( angleUnit );
 }
 
 QFont MarbleWidget::defaultFont() const
 {
-    return d->m_map->defaultFont();
+    return d->m_map.defaultFont();
 }
 
 void MarbleWidget::setDefaultFont( const QFont& font )
 {
-    d->m_map->setDefaultFont( font );
+    d->m_map.setDefaultFont( font );
 }
 
 void MarbleWidget::setSelection( const QRect& region )
@@ -1157,14 +1199,16 @@ QString MarbleWidget::distanceString() const
     qreal dist = distance();
     QString distanceUnitString;
 
-    const DistanceUnit distanceUnit = MarbleGlobal::getInstance()->locale()->distanceUnit();
+    const QLocale::MeasurementSystem measurementSystem = MarbleGlobal::getInstance()->locale()->measurementSystem();
 
-    if ( distanceUnit == Meter ) {
+    switch ( measurementSystem ) {
+    case QLocale::MetricSystem:
         distanceUnitString = tr("km");
-    }
-    else {
+        break;
+    case QLocale::ImperialSystem:
         dist *= KM2MI;
         distanceUnitString = tr("mi");
+        break;
     }
 
     return QString( "%L1 %2" ).arg( dist, 8, 'f', 1, QChar(' ') ).arg( distanceUnitString );
@@ -1193,7 +1237,7 @@ void MarbleWidget::setInputEnabled( bool enabled )
 
 QList<RenderPlugin *> MarbleWidget::renderPlugins() const
 {
-    return d->m_map->renderPlugins();
+    return d->m_map.renderPlugins();
 }
 
 void MarbleWidget::readPluginSettings( QSettings& settings )
@@ -1232,12 +1276,12 @@ void MarbleWidget::writePluginSettings( QSettings& settings ) const
 
 QList<AbstractFloatItem *> MarbleWidget::floatItems() const
 {
-    return d->m_map->floatItems();
+    return d->m_map.floatItems();
 }
 
 AbstractFloatItem * MarbleWidget::floatItem( const QString &nameId ) const
 {
-    return d->m_map->floatItem( nameId );
+    return d->m_map.floatItem( nameId );
 }
 
 void MarbleWidget::changeEvent( QEvent * event )
@@ -1257,42 +1301,40 @@ void MarbleWidget::flyTo( const GeoDataLookAt &newLookAt, FlyToMode mode )
         qreal const zoom = d->zoom( radius );
         // Prevent exceeding zoom range. Note: Bounding to range is not useful here
         if ( zoom >= minimumZoom() && zoom <= maximumZoom() ) {
-            d->m_map->setRadius( radius );
+            d->m_map.setRadius( radius );
             d->m_logzoom = qRound( d->zoom( radius ) );
 
             GeoDataCoordinates::Unit deg = GeoDataCoordinates::Degree;
-            d->m_map->centerOn( newLookAt.longitude( deg ), newLookAt.latitude( deg ) );
+            d->m_map.centerOn( newLookAt.longitude( deg ), newLookAt.latitude( deg ) );
 
             emit zoomChanged( d->m_logzoom );
             emit distanceChanged( distanceString() );
-            emit visibleLatLonAltBoxChanged( d->m_map->viewport()->viewLatLonAltBox() );
+            emit visibleLatLonAltBoxChanged( d->m_map.viewport()->viewLatLonAltBox() );
 
             d->repaint();
         }
     }
     else {
-        d->m_physics->flyTo( newLookAt, mode );
+        d->m_physics.flyTo( newLookAt, mode );
     }
 }
 
 void MarbleWidget::reloadMap()
 {
-    d->m_map->reload();
+    d->m_map.reload();
 }
 
 void MarbleWidget::downloadRegion( QString const & sourceDir, QVector<TileCoordsPyramid> const & pyramid )
 {
-    d->m_map->downloadRegion( sourceDir, pyramid );
+    d->m_map.downloadRegion( sourceDir, pyramid );
 }
 
 GeoDataLookAt MarbleWidget::lookAt() const
 {
     GeoDataLookAt result;
-    qreal lon( 0.0 ), lat( 0.0 );
 
-    d->m_map->viewport()->centerCoordinates( lon, lat );
-    result.setLongitude( lon );
-    result.setLatitude( lat );
+    result.setLongitude( d->m_map.viewport()->centerLongitude() );
+    result.setLatitude( d->m_map.viewport()->centerLatitude() );
     result.setAltitude( 0.0 );
     result.setRange( distance() * KM2METER );
 
