@@ -25,7 +25,6 @@
 
 #include "GeoDataCoordinates.h"
 #include "GeoDataLookAt.h"
-#include "Quaternion.h"
 #include "global.h"             // types needed in all of marble.
 #include "marble_export.h"
 #include "GeoDataFolder.h"
@@ -45,9 +44,11 @@ class GeoPainter;
 class GeoSceneDocument;
 class LayerInterface;
 class MarbleModel;
+class MarbleWidgetPopupMenu;
 class MarbleWidgetInputHandler;
 class MarbleWidgetPrivate;
 class MeasureTool;
+class Quaternion;
 class RenderPlugin;
 class RoutingLayer;
 class TextureLayer;
@@ -127,6 +128,9 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
     Q_PROPERTY(bool showGrid     READ showGrid        WRITE setShowGrid)
 
     Q_PROPERTY(bool showClouds   READ showClouds      WRITE setShowClouds)
+    Q_PROPERTY(bool showSunShading READ showSunShading WRITE setShowSunShading)
+    Q_PROPERTY(bool showCityLights READ showCityLights WRITE setShowCityLights)
+    Q_PROPERTY(bool showSunInZenith READ showSunInZenith WRITE setShowSunInZenith)
     Q_PROPERTY(bool showAtmosphere READ showAtmosphere WRITE setShowAtmosphere)
     Q_PROPERTY(bool showCrosshairs READ showCrosshairs WRITE setShowCrosshairs)
 
@@ -136,7 +140,6 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
     Q_PROPERTY(bool showOtherPlaces READ showOtherPlaces WRITE setShowOtherPlaces)
 
     Q_PROPERTY(bool showRelief   READ showRelief      WRITE setShowRelief)
-    Q_PROPERTY(bool showElevationModel READ showElevationModel WRITE setShowElevationModel)
 
     Q_PROPERTY(bool showIceLayer READ showIceLayer    WRITE setShowIceLayer)
     Q_PROPERTY(bool showBorders  READ showBorders     WRITE setShowBorders)
@@ -170,6 +173,8 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
     ViewportParams *viewport();
     const ViewportParams *viewport() const;
 
+    MarbleWidgetPopupMenu *popupMenu();
+
     /**
      * Returns the current input handler
      */
@@ -268,15 +273,10 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     QString mapThemeId() const;
 
-    /**
-     * @brief Return the active region in which it's possible to drag the view using the mouse.
-     */
-    QRegion activeRegion();
-
     /**
      * @brief Return the projected region which describes the (shape of the) projected surface.
      */
-    QRegion mapRegion();
+    QRegion mapRegion() const;
 
     /**
      * @brief  Return the radius of the globe in pixels.
@@ -337,7 +337,7 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      *         @c false if the geographical coordinates are not visible on the screen
      */
     bool screenCoordinates( qreal lon, qreal lat,
-                            qreal& x, qreal& y );
+                            qreal& x, qreal& y ) const;
 
     /**
      * @brief Get the earth coordinates corresponding to a pixel in the widget.
@@ -350,7 +350,7 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     bool geoCoordinates( int x, int y,
                          qreal& lon, qreal& lat,
-                         GeoDataCoordinates::Unit = GeoDataCoordinates::Degree );
+                         GeoDataCoordinates::Unit = GeoDataCoordinates::Degree ) const;
 
     /**
      * @brief Return the longitude of the center point.
@@ -368,13 +368,7 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      * @brief  Return how much the map will move if one of the move slots are called.
      * @return The move step.
      */
-    qreal moveStep();
-
-    /**
-     * @brief  Return the quaternion that specifies the rotation of the globe.
-     * @return The quaternion that describes the rotation of the globe.
-     */
-    Quaternion planetAxis() const;
+    qreal moveStep() const;
 
     /**
     * @brief Return the lookAt
@@ -437,6 +431,24 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     bool showClouds() const;
 
+    /**
+     * @brief  Return whether the night shadow is visible.
+     * @return visibility of night shadow
+     */
+    bool showSunShading() const;
+
+    /**
+     * @brief  Return whether the city lights are shown instead of the night shadow.
+     * @return visibility of city lights
+     */
+    bool showCityLights() const;
+
+    /**
+     * @brief  Return whether the sun is shown in the zenith.
+     * @return visibility of sun in the zenith
+     */
+    bool showSunInZenith() const;
+
     /**
      * @brief  Return whether the atmospheric glow is visible.
      * @return The cloud cover visibility.
@@ -485,12 +497,6 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     bool showRelief() const;
 
-    /**
-     * @brief  Return whether the elevation model is visible.
-     * @return The elevation model visibility.
-     */
-    bool showElevationModel() const;
-
     /**
      * @brief  Return whether the ice layer is visible.
      * @return The ice layer visibility.
@@ -515,12 +521,6 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     bool showLakes() const;
 
-    /**
-     * @brief Return whether Gps Data is visible.
-     * @return The Gps Data's visibility.
-     */
-    bool showGps() const;
-
     /**
      * @brief  Return whether the frame rate gets displayed.
      * @return the frame rates visibility
@@ -532,7 +532,7 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
     /**
      * @brief Retrieve the map quality depending on the view context
      */
-    MapQuality mapQuality( ViewContext = Still );
+    MapQuality mapQuality( ViewContext = Still ) const;
 
     /**
      * @brief Retrieve whether travels to a point should get animated
@@ -611,12 +611,6 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     void rotateBy( const qreal deltaLon, const qreal deltaLat, FlyToMode mode = Instant );
 
-        /**
-     * @brief  Rotate the view by the angle specified by a Quaternion.
-     * @param  incRot a quaternion specifying the rotation
-     */
-    void rotateBy( const Quaternion& incRot );
-
     /**
      * @brief  Center the view on a geographical point
      * @param  lat  an angle in degrees parallel to the latitude lines
@@ -647,6 +641,14 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     void centerOn( const GeoDataLatLonBox& box, bool animated = false );
 
+    /**
+     * @brief Center the view on a placemark according to the following logic:
+     * - if the placemark has a lookAt, zoom and center on that lookAt
+     * - otherwise use the placemark geometry's latLonAltBox
+     * @param box The GeoDataPlacemark to zoom and move the MarbleWidget to.
+     */
+    void centerOn( const GeoDataPlacemark& placemark, bool animated = false );
+
     /**
      * @brief  Set the latitude for the center point
      * @param  lat  the new value for the latitude in degree.
@@ -758,6 +760,24 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     void setShowClouds( bool visible );
 
+    /**
+     * @brief  Set whether the night shadow is visible.
+     * @param  visibile visibility of shadow
+     */
+    void setShowSunShading( bool visible );
+
+    /**
+     * @brief  Set whether city lights instead of night shadow are visible.
+     * @param  visible visibility of city lights
+     */
+    void setShowCityLights( bool visible );
+
+    /**
+     * @brief Set whether the sun is visible in the zenith.
+     * @param visible  visibility of the sun in the zenith
+     */
+    void setShowSunInZenith( bool visible );
+
     /**
      * @brief  Set whether the atmospheric glow is visible
      * @param  visible  visibility of the atmospheric glow
@@ -806,12 +826,6 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     void setShowRelief( bool visible );
 
-    /**
-     * @brief  Set whether the elevation model is visible
-     * @param  visible  visibility of the elevation model
-     */
-    void setShowElevationModel( bool visible );
-
     /**
      * @brief  Set whether the ice layer is visible
      * @param  visible  visibility of the ice layer
@@ -836,12 +850,6 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     void setShowLakes( bool visible );
 
-    /**
-     * @brief Set whether Gps Data is visible
-     * @param visible  visibility of the Gps Data
-     */
-    void setShowGps( bool visible );
-
     /**
      * @brief Set whether the frame rate gets shown
      * @param visible  visibility of the frame rate
@@ -917,16 +925,6 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
 
     //@}
 
-    /// @name Display update slots
-    //@{
-
-    /**
-     * Schedule repaint
-     */
-    void scheduleRepaint( const QRegion& dirtyRegion );
-
-    //@}
-
     /// @name Miscellaneous slots
     //@{
 
@@ -1016,13 +1014,22 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
     /**
      * @brief Enables custom drawing onto the MarbleWidget straight after
      * @brief the globe and before all other layers has been rendered.
-     * @param painter 
+     * @param painter
+     *
+     * @deprecated implement LayerInterface and add it using @p addLayer()
      */
     virtual void customPaint( GeoPainter *painter );
 
+ private:
+    Q_PRIVATE_SLOT( d, void updateMapTheme() )
+
  private:
     Q_DISABLE_COPY( MarbleWidget )
     MarbleWidgetPrivate  * const d;
+    friend class MarbleWidgetPrivate;
+
+    class CustomPaintLayer;
+    friend class CustomPaintLayer;
 
     friend class DownloadRegionDialog;
     TextureLayer *textureLayer();
