@@ -29,6 +29,8 @@
   #include <QtGui/QPinchGesture>
 #endif
 
+#include "kineticmodel.h"
+
 #include "global.h"
 #include "MarbleDebug.h"
 #include "GeoDataCoordinates.h"
@@ -58,6 +60,7 @@ public:
     QTimer *m_mouseWheelTimer;
     Qt::MouseButtons m_disabledMouseButtons;
     qreal m_wheelZoomTargetDistance;
+    bool m_panViaArrowsEnabled;
 };
 
 MarbleWidgetInputHandler::Protected::Protected( MarbleWidget *widget )
@@ -66,7 +69,8 @@ MarbleWidgetInputHandler::Protected::Protected( MarbleWidget *widget )
       m_positionSignalConnected( false ),
       m_mouseWheelTimer( 0 ),
       m_disabledMouseButtons( Qt::NoButton ),
-      m_wheelZoomTargetDistance( 0.0 )
+      m_wheelZoomTargetDistance( 0.0 ),
+      m_panViaArrowsEnabled( true )
 {
 }
 
@@ -118,11 +122,20 @@ bool MarbleWidgetInputHandler::isMouseButtonPopupEnabled( Qt::MouseButton mouseB
     return !( d->m_disabledMouseButtons & mouseButton );
 }
 
+void MarbleWidgetInputHandler::setPanViaArrowsEnabled( bool enabled )
+{
+    d->m_panViaArrowsEnabled = enabled;
+}
+
+bool MarbleWidgetInputHandler::panViaArrowsEnabled() const
+{
+    return d->m_panViaArrowsEnabled;
+}
 
 class MarbleWidgetDefaultInputHandler::Private
 {
  public:
-    Private();
+    Private( MarbleWidget *widget );
     ~Private();
 
     /**
@@ -165,7 +178,7 @@ class MarbleWidgetDefaultInputHandler::Private
     int m_leftPressedY;
     // The mouse pointer y position when the middle mouse button has been pressed.
     int m_midPressedY;
-    int m_radiusWhenPressed;
+    int m_startingRadius;
     // The center longitude in radian when the left mouse button has been pressed.
     qreal m_leftPressedLon;
     // The center latitude in radian when the left mouse button has been pressed.
@@ -174,25 +187,26 @@ class MarbleWidgetDefaultInputHandler::Private
     int m_dragThreshold;
     QTimer m_lmbTimer;
 
+    // Models to handle the kinetic spinning.
+    KineticModel m_kineticModel;
+
     QPoint m_selectionOrigin;
-    QRubberBand *m_selectionRubber;
+    QRubberBand m_selectionRubber;
 
     QPointer<AbstractDataPluginItem> m_lastToolTipItem;
     QTimer m_toolTipTimer;
     QPoint m_toolTipPosition;
 
-    MarbleWidgetPopupMenu *m_popupmenu;
-
     QAction  *m_addMeasurePointAction;
     QAction  *m_removeLastMeasurePointAction;
     QAction  *m_removeMeasurePointsAction;
 };
 
-MarbleWidgetDefaultInputHandler::Private::Private()
+MarbleWidgetDefaultInputHandler::Private::Private( MarbleWidget *widget )
     : m_leftPressed( false ),
       m_midPressed( false ),
       m_dragThreshold( MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen ? 15 : 3 ),
-      m_popupmenu( 0 )
+      m_selectionRubber( QRubberBand::Rectangle, widget )
 {
     m_curpmtl.load( MarbleDirs::path("bitmaps/cursor_tl.xpm") );
     m_curpmtc.load( MarbleDirs::path("bitmaps/cursor_tc.xpm") );
@@ -238,7 +252,7 @@ void MarbleWidgetDefaultInputHandler::Private::ZoomAt(MarbleWidget* marbleWidget
 
     ViewportParams soon;
     soon.setProjection(now->projection());
-    soon.setPlanetAxis(now->planetAxis());
+    soon.centerOn(now->centerLongitude(), now->centerLatitude());
     soon.setSize(now->size());
 
     qreal newRadius = marbleWidget->radiusFromDistance(newDistance);
@@ -286,11 +300,14 @@ void MarbleWidgetDefaultInputHandler::Private::MoveTo( MarbleWidget* marbleWidge
 }
 
 MarbleWidgetDefaultInputHandler::MarbleWidgetDefaultInputHandler( MarbleWidget *widget )
-    : MarbleWidgetInputHandler( widget ), d( new Private )
+    : MarbleWidgetInputHandler( widget ),
+      d( new Private( widget ) )
 {
-    d->m_selectionRubber = new QRubberBand( QRubberBand::Rectangle,
-                                            MarbleWidgetInputHandler::d->m_widget );
-    d->m_selectionRubber->hide();
+#if QT_VERSION >= 0x40600
+    widget->grabGesture( Qt::PinchGesture );
+#endif
+
+    d->m_selectionRubber.hide();
 
     d->m_toolTipTimer.setSingleShot( true );
     d->m_toolTipTimer.setInterval( TOOLTIP_START_INTERVAL );
@@ -298,26 +315,28 @@ MarbleWidgetDefaultInputHandler::MarbleWidgetDefaultInputHandler( MarbleWidget *
     d->m_lmbTimer.setSingleShot(true);
     connect( &d->m_lmbTimer, SIGNAL(timeout()), this, SLOT(lmbTimeout()));
 
+    d->m_kineticModel.setUpdateInterval( 35 );
+    connect( &d->m_kineticModel, SIGNAL( positionChanged() ), SLOT( adjustCenter() ) );
+    connect( &d->m_kineticModel, SIGNAL( finished() ), SLOT( restoreViewContext() ) );
+
     // The interface to the measure tool consists of a RMB popup menu
     // and some signals.
     MeasureTool *measureTool = MarbleWidgetInputHandler::d->m_widget->measureTool();
 
     // Connect the inputHandler and the measure tool to the popup menu
-    if ( !d->m_popupmenu ) {
-        d->m_popupmenu = new MarbleWidgetPopupMenu( MarbleWidgetInputHandler::d->m_widget,
-                                                    MarbleWidgetInputHandler::d->m_model );
-    }
-
     d->m_addMeasurePointAction = new QAction( tr( "Add &Measure Point" ), this);
-    d->m_popupmenu->addAction( Qt::RightButton, d->m_addMeasurePointAction );
     d->m_removeLastMeasurePointAction = new QAction( tr( "Remove &Last Measure Point" ),
                                                   this);
     d->m_removeLastMeasurePointAction->setEnabled(false);
-    d->m_popupmenu->addAction( Qt::RightButton, d->m_removeLastMeasurePointAction );
     d->m_removeMeasurePointsAction = new QAction( tr( "&Remove Measure Points" ),
                                                 this);
     d->m_removeMeasurePointsAction->setEnabled(false);
-    d->m_popupmenu->addAction( Qt::RightButton, d->m_removeMeasurePointsAction );
+
+    if ( ! MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen ) {
+        MarbleWidgetInputHandler::d->m_widget->popupMenu()->addAction( Qt::RightButton, d->m_addMeasurePointAction );
+        MarbleWidgetInputHandler::d->m_widget->popupMenu()->addAction( Qt::RightButton, d->m_removeLastMeasurePointAction );
+        MarbleWidgetInputHandler::d->m_widget->popupMenu()->addAction( Qt::RightButton, d->m_removeMeasurePointsAction );
+    }
 
     connect( d->m_addMeasurePointAction, SIGNAL( triggered() ),
              SLOT( addMeasurePoint() ) );
@@ -336,14 +355,12 @@ MarbleWidgetDefaultInputHandler::MarbleWidgetDefaultInputHandler( MarbleWidget *
 
 MarbleWidgetDefaultInputHandler::~MarbleWidgetDefaultInputHandler()
 {
-    // FIXME: move to Private
-    delete d->m_selectionRubber;
     delete d;
 }
 
 void MarbleWidgetDefaultInputHandler::lmbTimeout()
 {
-    if (!d->m_selectionRubber->isVisible()) {
+    if (!d->m_selectionRubber.isVisible()) {
         emit lmbRequest( d->m_leftPressedX, d->m_leftPressedY );
     }
 }
@@ -367,7 +384,8 @@ void MarbleWidgetInputHandler::installPluginEventFilter( RenderPlugin *renderPlu
 void MarbleWidgetDefaultInputHandler::showLmbMenu( int x, int y )
 {
     if ( isMouseButtonPopupEnabled( Qt::LeftButton ) ) {
-        d->m_popupmenu->showLmbMenu( x, y );
+        MarbleWidgetInputHandler::d->m_widget->popupMenu()->showLmbMenu( x, y );
+        d->m_toolTipTimer.stop();
     }
 }
 
@@ -375,7 +393,7 @@ void MarbleWidgetDefaultInputHandler::showRmbMenu( int x, int y )
 {
     if ( isMouseButtonPopupEnabled( Qt::RightButton ) ) {
         d->m_addMeasurePointAction->setData( QPoint( x, y ) );
-        d->m_popupmenu->showRmbMenu( x, y );
+        MarbleWidgetInputHandler::d->m_widget->popupMenu()->showRmbMenu( x, y );
     }
 }
 
@@ -393,12 +411,12 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
 {
     Q_UNUSED( o );
 
-    if (d->m_selectionRubber->isVisible() && e->type() == QEvent::MouseMove)
+    if (d->m_selectionRubber.isVisible() && e->type() == QEvent::MouseMove)
     {
         QMouseEvent *event = static_cast<QMouseEvent*>( e );
         if (!(event->modifiers() & Qt::ControlModifier))
         {
-            d->m_selectionRubber->hide();
+            d->m_selectionRubber.hide();
         }
     }
 
@@ -421,7 +439,6 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
          || e->type() == QEvent::MouseButtonRelease )
     {
         QMouseEvent *event = static_cast<QMouseEvent*>( e );
-        QRegion activeRegion = MarbleWidgetInputHandler::d->m_widget->activeRegion();
 
         int dirX = 0;
         int dirY = 0;
@@ -430,7 +447,11 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
         if ( event->type() == QEvent::MouseMove
              && !( event->buttons() & Qt::LeftButton ) )
         {
-            d->m_leftPressed = false;
+            if ( d->m_leftPressed ) {
+                d->m_leftPressed = false;
+
+                d->m_kineticModel.release();
+            }
         }
         if ( event->type() == QEvent::MouseMove
              && !( event->buttons() & Qt::MidButton ) )
@@ -442,7 +463,7 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
         // that occur above visible float items. Mouse motion events are still
         // handled, however.
         if ( e->type() != QEvent::MouseMove
-             && !d->m_selectionRubber->isVisible() )
+             && !d->m_selectionRubber.isVisible() )
         {
             foreach ( AbstractFloatItem *floatItem, MarbleWidgetInputHandler::d->m_widget->floatItems() ) {
                 if ( floatItem->enabled() && floatItem->visible()
@@ -454,27 +475,28 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
             }
         }
 
+        qreal mouseLon;
+        qreal mouseLat;
+        const bool isMouseAboveMap = MarbleWidgetInputHandler::d->m_widget->geoCoordinates( event->x(), event->y(),
+                                                 mouseLon, mouseLat,
+                                                 GeoDataCoordinates::Radian );
+
         // emit the position string only if the signal got attached
         if ( MarbleWidgetInputHandler::d->m_positionSignalConnected ) {
-            qreal lat;
-            qreal lon;
-            bool isValid = MarbleWidgetInputHandler::d->m_widget->geoCoordinates( event->x(), event->y(),
-                                                     lon, lat,
-                                                     GeoDataCoordinates::Radian );
-
-            if ( !isValid ) {
+            if ( !isMouseAboveMap ) {
                 emit mouseMoveGeoPosition( tr( NOT_AVAILABLE ) );
             }
             else {
-                QString position = GeoDataCoordinates( lon, lat ).toString();
+                QString position = GeoDataCoordinates( mouseLon, mouseLat ).toString();
                 emit mouseMoveGeoPosition( position );
             }
         }
 
+        QPoint mousePosition( event->x(), event->y() );
 
-        if ( activeRegion.contains( event->pos() ) || d->m_selectionRubber->isVisible() ) {
-
-            // Regarding mouse button presses:
+        if ( isMouseAboveMap || d->m_selectionRubber.isVisible()
+             || MarbleWidgetInputHandler::d->m_widget->whichFeatureAt( mousePosition ).size() != 0) {
+            
             if ( e->type() == QEvent::MouseButtonPress
                  && event->button() == Qt::LeftButton ) {
 
@@ -483,7 +505,7 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
 
                 d->m_leftPressed = true;
                 d->m_midPressed = false;
-                d->m_selectionRubber->hide();
+                d->m_selectionRubber.hide();
 
                 // On the single event of a mouse button press these
                 // values get stored, to enable us to e.g. calculate the
@@ -496,8 +518,12 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                 d->m_leftPressedLon = MarbleWidgetInputHandler::d->m_widget->centerLongitude() * DEG2RAD;
                 d->m_leftPressedLat = MarbleWidgetInputHandler::d->m_widget->centerLatitude() * DEG2RAD;
 
+
                 d->m_leftPressedDirection = 1;
 
+                d->m_kineticModel.setPosition( RAD2DEG * ( qreal )( d->m_leftPressedLon ), RAD2DEG * ( qreal )( d->m_leftPressedLat ) );
+                d->m_kineticModel.resetSpeed();
+
                 // Choose spin direction by taking into account whether we
                 // drag above or below the visible pole.
                 if ( MarbleWidgetInputHandler::d->m_widget->projection() == Spherical ) {
@@ -524,10 +550,12 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                  && event->button() == Qt::MidButton ) {
                 d->m_midPressed = true;
                 d->m_leftPressed = false;
-                d->m_radiusWhenPressed = MarbleWidgetInputHandler::d->m_widget->radius();
+                d->m_startingRadius = MarbleWidgetInputHandler::d->m_widget->radius();
                 d->m_midPressedY = event->y();
 
-                d->m_selectionRubber->hide();
+                d->m_kineticModel.release();
+
+                d->m_selectionRubber.hide();
                 MarbleWidgetInputHandler::d->m_widget->setViewContext( Animation );
             }
 
@@ -542,9 +570,9 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
             {
                 qDebug("Marble: Starting selection");
                 d->m_lmbTimer.stop();                
-                d->m_selectionOrigin = event->globalPos();
-                d->m_selectionRubber->setGeometry( QRect( d->m_selectionOrigin, QSize() ));
-                d->m_selectionRubber->show();
+                d->m_selectionOrigin = event->pos();
+                d->m_selectionRubber.setGeometry( QRect( d->m_selectionOrigin, QSize() ));
+                d->m_selectionRubber.show();
             }
 
             // Regarding mouse button releases:
@@ -556,9 +584,8 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                 //as requested
                 emit mouseClickScreenPosition( d->m_leftPressedX, d->m_leftPressedY );
 
-                MarbleWidgetInputHandler::d->m_widget->setViewContext( Still );
-
                 d->m_leftPressed = false;
+                d->m_kineticModel.release();
             }
 
             if ( e->type() == QEvent::MouseButtonRelease
@@ -574,17 +601,15 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
 
             if ( e->type() == QEvent::MouseButtonRelease
                 && event->button() == Qt::LeftButton 
-                 && d->m_selectionRubber->isVisible() ) 
+                 && d->m_selectionRubber.isVisible() )
             {
                 qDebug("Marble: Leaving selection");
-                QRect r( MarbleWidgetInputHandler::d->m_widget->mapFromGlobal( d->m_selectionRubber->geometry().topLeft() ),
-                         MarbleWidgetInputHandler::d->m_widget->mapFromGlobal( d->m_selectionRubber->geometry().bottomRight() ));
-                MarbleWidgetInputHandler::d->m_widget->setSelection( r );
-                d->m_selectionRubber->hide();
+                MarbleWidgetInputHandler::d->m_widget->setSelection( d->m_selectionRubber.geometry() );
+                d->m_selectionRubber.hide();
             }
 
             // Regarding all kinds of mouse moves:
-            if ( d->m_leftPressed && !d->m_selectionRubber->isVisible() ) {
+            if ( d->m_leftPressed && !d->m_selectionRubber.isVisible() ) {
                 qreal radius = ( qreal )( MarbleWidgetInputHandler::d->m_widget->radius() );
                 int deltax = event->x() - d->m_leftPressedX;
                 int deltay = event->y() - d->m_leftPressedY;
@@ -593,10 +618,10 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                      || abs( deltay ) > d->m_dragThreshold ) {
 
                     d->m_lmbTimer.stop();
-                    MarbleWidgetInputHandler::d->m_widget->centerOn( RAD2DEG * ( qreal )( d->m_leftPressedLon )
-                                                                     - 90.0 * d->m_leftPressedDirection * deltax / radius,
-                                                                     RAD2DEG * ( qreal )( d->m_leftPressedLat )
-                                                                     + 90.0 * deltay / radius );
+
+                    qreal posLon = RAD2DEG * ( qreal )( d->m_leftPressedLon ) - 90.0 * d->m_leftPressedDirection * deltax / radius;
+                    qreal posLat = RAD2DEG * ( qreal )( d->m_leftPressedLat ) + 90.0 * deltay / radius;
+                    d->m_kineticModel.setPosition( posLon, posLat );
                 }
             }
 
@@ -604,19 +629,20 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
             if ( d->m_midPressed ) {
                 int eventy = event->y();
                 int dy = d->m_midPressedY - eventy;
-                MarbleWidgetInputHandler::d->m_widget->setRadius( d->m_radiusWhenPressed * pow( 1.005, dy ) );
+                MarbleWidgetInputHandler::d->m_widget->setRadius( d->m_startingRadius * pow( 1.005, dy ) );
             }
 
-            if ( d->m_selectionRubber->isVisible() ) 
+            if ( d->m_selectionRubber.isVisible() )
             {
                 // We change selection.
-                d->m_selectionRubber->setGeometry( QRect( d->m_selectionOrigin,
-                                                          event->globalPos() ).normalized() );
+                d->m_selectionRubber.setGeometry( QRect( d->m_selectionOrigin, event->pos() ).normalized() );
             }
         }
         else {
             d->m_leftPressed = false;
 
+            d->m_kineticModel.release();
+
             QRect boundingRect = MarbleWidgetInputHandler::d->m_widget->mapRegion().boundingRect();
 
             if ( boundingRect.width() != 0 ) {
@@ -638,7 +664,7 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                 dirY = -1;
 
             if ( event->button() == Qt::LeftButton
-                 && e->type() == QEvent::MouseButtonPress ) {
+                 && e->type() == QEvent::MouseButtonPress && panViaArrowsEnabled() ) {
 
                 d->m_lmbTimer.stop();                
                 if ( polarity < 0 )
@@ -652,8 +678,6 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
 
         // Adjusting Cursor shape
 
-        QPoint mousePosition( event->x(), event->y() );
-        
         // Find out if there are data items and if one has defined an action
         QList<AbstractDataPluginItem *> dataItems
             = MarbleWidgetInputHandler::d->m_widget->whichItemAt( mousePosition );
@@ -700,7 +724,11 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                 d->m_arrowCur [1][1] = QCursor( Qt::PointingHandCursor );
         }
 
-        MarbleWidgetInputHandler::d->m_widget->setCursor( d->m_arrowCur[dirX+1][dirY+1] );
+        if ( panViaArrowsEnabled() ) {
+            MarbleWidgetInputHandler::d->m_widget->setCursor( d->m_arrowCur[dirX+1][dirY+1] );
+        } else {
+            MarbleWidgetInputHandler::d->m_widget->setCursor( d->m_arrowCur [1][1] );
+        }
 
         // let others, especially float items, still process the event
         // Mouse move events need to be eaten to avoid the default oxygen behavior of
@@ -740,8 +768,6 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
 
                 MarbleWidget *marbleWidget = MarbleWidgetInputHandler::d->m_widget;
 
-                marbleWidget->setViewContext( Animation );
-
                 bool isValid = marbleWidget->geoCoordinates(center.x(), center.y(),
                              destLon, destLat, GeoDataCoordinates::Radian );
 
@@ -749,9 +775,28 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                     marbleWidget->viewport()->setFocusPoint(GeoDataCoordinates(destLon, destLat));
                 }
 
-                //convert the scaleFactor to be 0: the same: < 0: smaller, > 0: bigger and make it bigger by multiplying for an arbitrary big value
-               marbleWidget->zoomViewBy( (scaleFactor-1)*200); 
-               return true;
+                switch ( pinch->state() ) {
+                case Qt::NoGesture:
+                    break;
+                case Qt::GestureStarted:
+                    marbleWidget->setViewContext( Animation );
+                    d->m_startingRadius = marbleWidget->radius();
+                    break;
+                case Qt::GestureUpdated:
+                    marbleWidget->setRadius( marbleWidget->radius() * scaleFactor );
+                    break;
+                case Qt::GestureFinished:
+                    marbleWidget->viewport()->resetFocusPoint();
+                    marbleWidget->setViewContext( Still );
+                    break;
+                case Qt::GestureCanceled:
+                    marbleWidget->setRadius( d->m_startingRadius );
+                    marbleWidget->viewport()->resetFocusPoint();
+                    marbleWidget->setViewContext( Still );
+                    break;
+                }
+
+                return true;
             }
         }
         else
@@ -817,6 +862,12 @@ void MarbleWidgetDefaultInputHandler::setNumberOfMeasurePoints( int newNumber )
     d->m_removeLastMeasurePointAction->setEnabled(enableMeasureActions);
 }
 
+void MarbleWidgetDefaultInputHandler::adjustCenter()
+{
+    MarbleWidgetInputHandler::d->m_widget->centerOn( d->m_kineticModel.position().x(), d->m_kineticModel.position().y() );
+}
+
 }
 
 #include "MarbleWidgetInputHandler.moc"
+
