@@ -21,6 +21,7 @@
 #include "GeoDataPlacemark.h"
 #include "GeoDataStyle.h"
 #include "PlacemarkInfoDialog.h"
+#include "MarbleClock.h"
 #include "MarbleDebug.h"
 #include "Planet.h"
 #include "routing/RoutingManager.h"
@@ -40,67 +41,72 @@ using namespace Marble;
 /* TRANSLATOR Marble::MarbleWidgetPopupMenu */
 
 MarbleWidgetPopupMenu::MarbleWidgetPopupMenu(MarbleWidget *widget, 
-                                         MarbleModel *model)
+                                         const MarbleModel *model)
     : QObject(widget),
       m_model(model),
       m_widget(widget),
       m_lmbMenu( new QMenu( m_widget ) ),
       m_rmbMenu( new QMenu( m_widget ) ),
-      m_smallScreenMenu( new QMenu( m_widget ) ),
+      m_copyCoordinateAction( new QAction( tr("Copy Coordinates"), this ) ),
+      m_rmbExtensionPoint( 0 ),
       m_runnerManager( 0 )
-{
-    connect( m_lmbMenu, SIGNAL( triggered( QAction* ) ),
-             this,      SLOT( showFeatureInfo( QAction* ) ) );
-    createActions();
-}
-
-void MarbleWidgetPopupMenu::createActions()
 {
     //	Property actions (Left mouse button)
-    //	m_planetAction = new QAction(QIcon("icon.png"), tr("&Earth"), this);
-    m_planetAction = new QAction( tr( "&Earth" ), this );
-    m_planetAction->setData( 0 );
-    m_copyCoordinateAction = new QAction( tr( "0 N 0 W" ), this );
+    m_infoDialogAction = new QAction( this );
+    m_infoDialogAction->setData( 0 );
+
+    m_showOrbitAction = new QAction( tr( "Display orbit" ), this );
+    m_showOrbitAction->setCheckable( true );
+    m_showOrbitAction->setData( 0 );
+
+    m_trackPlacemarkAction = new QAction( tr( "Keep centered" ), this );
+    m_trackPlacemarkAction->setData( 0 );
 
     //	Tool actions (Right mouse button)
     QAction* fromHere = new QAction( tr( "Directions &from here" ), this );
     QAction* toHere = new QAction( tr( "Directions &to here" ), this );
-    m_setHomePointAction  = new QAction( tr( "&Set Home Location" ), this);
     QAction* addBookmark = new QAction( QIcon(":/icons/bookmark-new.png"),
                                         tr( "Add &Bookmark" ), this );
     QAction* fullscreenAction = new QAction( tr( "&Full Screen Mode" ), this );
     fullscreenAction->setCheckable( true );
 
-    m_aboutDialogAction = new QAction( tr( "&About" ), this );
+    QAction* aboutDialogAction = new QAction( tr( "&About" ), this );
 
     QMenu* infoBoxMenu = createInfoBoxMenu();
 
-    m_rmbExtensionPoint = m_rmbMenu->addSeparator();
+    const bool smallScreen = MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen;
+
+    if ( !smallScreen ) {
+        m_rmbExtensionPoint = m_rmbMenu->addSeparator();
+    }
+
     m_rmbMenu->addAction( fromHere );
     m_rmbMenu->addAction( toHere );
     m_rmbMenu->addSeparator();
-    m_rmbMenu->addAction( m_setHomePointAction );
-    m_rmbMenu->addAction( addBookmark );
+    m_rmbMenu->addAction( tr( "&Address Details" ), this, SLOT( startReverseGeocoding() ) );
+    if ( smallScreen ) {
+        m_rmbMenu->addAction( addBookmark );
+    }
+    else {
+        m_rmbMenu->addAction( m_copyCoordinateAction );
+    }
     m_rmbMenu->addSeparator();
     m_rmbMenu->addMenu( infoBoxMenu );
-    m_rmbMenu->addAction( m_aboutDialogAction );
 
-    m_smallScreenMenu->addAction( fromHere );
-    m_smallScreenMenu->addAction( toHere );
-    m_smallScreenMenu->addSeparator();
-    m_smallScreenMenu->addAction( m_setHomePointAction );
-    m_smallScreenMenu->addAction( addBookmark );
-    m_smallScreenMenu->addSeparator();
-    m_smallScreenMenu->addMenu( infoBoxMenu );
-    m_smallScreenMenu->addAction( fullscreenAction );
-    m_smallScreenMenu->addSeparator();
+    if ( !smallScreen ) {
+        m_rmbMenu->addAction( aboutDialogAction );
+    } else {
+        m_rmbMenu->addAction( fullscreenAction );
+    }
 
     connect( fromHere, SIGNAL( triggered( ) ), SLOT( directionsFromHere() ) );
     connect( toHere, SIGNAL( triggered( ) ), SLOT( directionsToHere() ) );
-    connect( m_setHomePointAction, SIGNAL( triggered() ), SLOT( slotSetHomePoint() ) );
     connect( addBookmark, SIGNAL( triggered( ) ), SLOT( addBookmark() ) );
-    connect( m_aboutDialogAction, SIGNAL( triggered() ), SLOT( slotAboutDialog() ) );
-    connect( m_copyCoordinateAction,SIGNAL( triggered() ), SLOT( slotCopyCoordinates() ) );
+    connect( aboutDialogAction, SIGNAL( triggered() ), SLOT( slotAboutDialog() ) );
+    connect( m_copyCoordinateAction, SIGNAL( triggered() ), SLOT( slotCopyCoordinates() ) );
+    connect( m_infoDialogAction, SIGNAL( triggered() ), SLOT( slotInfoDialog() ) );
+    connect( m_showOrbitAction, SIGNAL( triggered(bool) ), SLOT( slotShowOrbit(bool) ) );
+    connect( m_trackPlacemarkAction, SIGNAL( triggered(bool) ), SLOT( slotTrackPlacemark() ) );
     connect( fullscreenAction, SIGNAL( triggered( bool ) ), this, SLOT( toggleFullscreen( bool ) ) );
 }
 
@@ -120,19 +126,21 @@ QMenu* MarbleWidgetPopupMenu::createInfoBoxMenu()
 
 void MarbleWidgetPopupMenu::showLmbMenu( int xpos, int ypos )
 {
-    QPoint  curpos = QPoint( xpos, ypos ); 
-    m_setHomePointAction->setData( curpos );
     bool const smallScreen = MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen;
     if ( smallScreen ) {
-        m_smallScreenMenu->popup( m_widget->mapToGlobal( curpos ) );
+        showRmbMenu( xpos, ypos );
         return;
     }
 
     if (!m_lmbMenu->isEmpty()) {
+        m_lmbMenu->clear();
         // just clear()-ing the menu won't delete the submenus
-        m_lmbMenu->deleteLater();
-        m_lmbMenu = new QMenu( m_widget );
+        foreach( QObject *child, m_lmbMenu->children() ) {
+            child->deleteLater();
+        }
     }
+
+    const QPoint curpos = QPoint( xpos, ypos );
     m_featurelist = m_widget->whichFeatureAt( curpos );
 
     int  actionidx = 1;
@@ -141,13 +149,35 @@ void MarbleWidgetPopupMenu::showLmbMenu( int xpos, int ypos )
     QVector<const GeoDataPlacemark*>::const_iterator const itEnd = m_featurelist.constEnd();
     for (; it != itEnd; ++it )
     {
-        QAction *action = new QAction( (*it)->name(), m_lmbMenu );
-        action->setData( actionidx );
-        action->setIcon( (*it)->style()->iconStyle().icon() );
-        m_lmbMenu->addAction( action );
+        QString name = (*it)->name();
+        QPixmap icon = (*it)->style()->iconStyle().icon();
+        m_infoDialogAction->setData( actionidx );
+
+        //TODO: don't hardcode this check ?
+        if ( (*it)->visualCategory() != GeoDataFeature::Satellite ) {
+            m_infoDialogAction->setText( name );
+            m_infoDialogAction->setIcon( icon );
+
+            m_lmbMenu->addAction( m_infoDialogAction );
+        } else {
+            QMenu *subMenu = new QMenu( name, m_lmbMenu );
+            subMenu->setIcon( icon );
+            m_infoDialogAction->setText( tr( "Satellite information" ) );
+            m_infoDialogAction->setIcon( QIcon() );
+            subMenu->addAction( m_infoDialogAction );
+
+            m_showOrbitAction->setChecked( (*it)->style()->lineStyle().penStyle() != Qt::NoPen );
+            m_showOrbitAction->setData( actionidx );
+            subMenu->addAction( m_showOrbitAction );
+
+            m_trackPlacemarkAction->setData( actionidx );
+            subMenu->addAction( m_trackPlacemarkAction );
+
+            m_lmbMenu->addAction( subMenu->menuAction() );
+        }
         actionidx++;
     }
-    
+
     m_itemList = m_widget->whichItemAt( curpos );
     QList<AbstractDataPluginItem *>::const_iterator itW = m_itemList.constBegin();
     QList<AbstractDataPluginItem *>::const_iterator const itWEnd = m_itemList.constEnd();
@@ -156,73 +186,84 @@ void MarbleWidgetPopupMenu::showLmbMenu( int xpos, int ypos )
         m_lmbMenu->addAction( (*itW)->action() );
     }
 
-    // Not implemented yet ;-)
-    m_planetAction->setEnabled( false );
-
-    m_lmbMenu->addAction( m_planetAction );
-    m_lmbMenu->addSeparator();
-
-    // Setting the proper planet name
-    QString targetString = m_model->planet()->name();
-    mDebug() << "targetString" << targetString;
-    // FIXME: this should be removed later on ...
-    if ( targetString == "Earth" ) 
-        targetString = tr( "&Earth" );
-
-    m_planetAction->setText( targetString );
-
-    qreal  lat;
-    qreal  lon;
-
-    m_widget->geoCoordinates( xpos, ypos, lon, lat, GeoDataCoordinates::Radian );
-
-    m_copyCoordinateAction->setEnabled( true );
-    m_copyCoordinateAction->setText( tr("Copy Coordinates") );
-    m_copyCoordinateAction->setData( curpos );
-
-    QMenu *positionMenu = m_lmbMenu->addMenu( GeoDataCoordinates( lon, lat, GeoDataCoordinates::Radian ).toString() );
-    positionMenu->menuAction()->setFont( QFont( "Sans Serif", 7, 50, false ) );
-    positionMenu->addAction( m_copyCoordinateAction );
-    positionMenu->addAction( tr( "&Address Details" ), this, SLOT( startReverseGeocoding() ) );
-
-    m_lmbMenu->popup( m_widget->mapToGlobal( curpos ) );
+    if ( !m_lmbMenu->isEmpty() ) {
+        m_lmbMenu->popup( m_widget->mapToGlobal( curpos ) );
+    }
 }
 
 
 void MarbleWidgetPopupMenu::showRmbMenu( int xpos, int ypos )
 {
+    qreal lon, lat;
+    const bool visible = m_widget->geoCoordinates( xpos, ypos, lon, lat, GeoDataCoordinates::Radian );
+    if ( !visible )
+        return;
+
     QPoint curpos = QPoint( xpos, ypos );
-    m_setHomePointAction->setData( curpos );
+    m_copyCoordinateAction->setData( curpos );
+
     m_rmbMenu->popup( m_widget->mapToGlobal( curpos ) );
 }
 
-
-void MarbleWidgetPopupMenu::showFeatureInfo( QAction* action )
+void MarbleWidgetPopupMenu::slotInfoDialog()
 {
+    QAction *action = qobject_cast<QAction *>( sender() );
+    if ( action == 0 ) {
+        mDebug() << "Warning: slotInfoDialog should be called by a QAction signal";
+        return;
+    }
+
     int actionidx = action->data().toInt();
 
     if ( actionidx > 0 ) {
         const GeoDataPlacemark *index = m_featurelist.at( actionidx -1 );
 
-        QPointer<PlacemarkInfoDialog> dialog = new PlacemarkInfoDialog( index, m_widget );
+        QPointer<PlacemarkInfoDialog> dialog = new PlacemarkInfoDialog( index, m_widget->model()->clock(),  m_widget );
         dialog->setWindowModality( Qt::WindowModal );
         dialog->exec();
         delete dialog;
     }
 }
 
-void MarbleWidgetPopupMenu::slotSetHomePoint()
+void MarbleWidgetPopupMenu::slotShowOrbit( bool show )
 {
-    GeoDataCoordinates coordinates;
-    if ( mouseCoordinates( &coordinates, m_setHomePointAction ) ) {
-        m_widget->model()->setHome( coordinates, m_widget->zoom() );
+    QAction *action = qobject_cast<QAction *>( sender() );
+    if ( action == 0 ) {
+        mDebug() << "Warning: slotShowOrbit should be called by a QAction signal";
+        return;
+    }
+
+    int actionidx = action->data().toInt();
+
+    if ( actionidx > 0 ) {
+        const GeoDataPlacemark *index = m_featurelist.at( actionidx -1 );
+
+        Qt::PenStyle penStyle = show ? Qt::SolidLine : Qt::NoPen;
+        index->style()->lineStyle().setPenStyle( penStyle );
+    }
+}
+
+void MarbleWidgetPopupMenu::slotTrackPlacemark()
+{
+    QAction *action = qobject_cast<QAction *>( sender() );
+    if ( action == 0 ) {
+        mDebug() << "Warning: slotTrackPlacemark should be called by a QAction signal";
+        return;
+    }
+
+    int actionidx = action->data().toInt();
+
+    if ( actionidx > 0 ) {
+        mDebug() << actionidx;
+        const GeoDataPlacemark *index = m_featurelist.at( actionidx -1 );
+        emit trackPlacemark( index );
     }
 }
 
 void MarbleWidgetPopupMenu::slotCopyCoordinates()
 {
     GeoDataCoordinates coordinates;
-    if ( mouseCoordinates( &coordinates, m_setHomePointAction ) ) {
+    if ( mouseCoordinates( &coordinates, m_copyCoordinateAction ) ) {
 	const qreal longitude_degrees = coordinates.longitude(GeoDataCoordinates::Degree);
 	const qreal latitude_degrees = coordinates.latitude(GeoDataCoordinates::Degree);
 
@@ -287,7 +328,7 @@ void MarbleWidgetPopupMenu::directionsFromHere()
     if ( request )
     {
         GeoDataCoordinates coordinates;
-        if ( mouseCoordinates( &coordinates, m_setHomePointAction ) ) {
+        if ( mouseCoordinates( &coordinates, m_copyCoordinateAction ) ) {
             if ( request->size() > 0 ) {
                 request->setPosition( 0, coordinates );
             } else {
@@ -304,7 +345,7 @@ void MarbleWidgetPopupMenu::directionsToHere()
     if ( request )
     {
         GeoDataCoordinates coordinates;
-        if ( mouseCoordinates( &coordinates, m_setHomePointAction ) ) {
+        if ( mouseCoordinates( &coordinates, m_copyCoordinateAction ) ) {
             if ( request->size() > 1 ) {
                 request->setPosition( request->size()-1, coordinates );
             } else {
@@ -322,12 +363,17 @@ bool MarbleWidgetPopupMenu::mouseCoordinates( GeoDataCoordinates* coordinates, Q
         return false;
     }
 
-    QPoint p = dataContainer->data().toPoint();
-    qreal lat( 0.0 ), lon( 0.0 );
-    bool valid = m_widget->geoCoordinates( p.x(), p.y(), lon, lat, GeoDataCoordinates::Radian );
-    if ( valid )
-    {
-        *coordinates = GeoDataCoordinates( lon, lat );
+    bool valid = true;
+    if ( !m_featurelist.isEmpty() ) {
+        *coordinates = m_featurelist.first()->coordinate( m_model->clock()->dateTime() );
+    } else {
+        QPoint p = dataContainer->data().toPoint();
+        qreal lat( 0.0 ), lon( 0.0 );
+
+        valid = m_widget->geoCoordinates( p.x(), p.y(), lon, lat, GeoDataCoordinates::Radian );
+        if ( valid ) {
+            *coordinates = GeoDataCoordinates( lon, lat );
+        }
     }
 
     return valid;
@@ -358,7 +404,7 @@ void MarbleWidgetPopupMenu::showAddressInformation(const GeoDataCoordinates &, c
 void MarbleWidgetPopupMenu::addBookmark()
 {
     GeoDataCoordinates coordinates;
-    if ( mouseCoordinates( &coordinates, m_setHomePointAction ) ) {
+    if ( mouseCoordinates( &coordinates, m_copyCoordinateAction ) ) {
         QPointer<EditBookmarkDialog> dialog = new EditBookmarkDialog( m_widget->model()->bookmarkManager(), m_widget );
         GeoDataLookAt lookat = m_widget->lookAt();
         lookat.setCoordinates( coordinates );
