@@ -40,12 +40,13 @@ void PositionTrackingPrivate::setPosition( GeoDataCoordinates position,
         PositionProviderStatusAvailable )
     {
         GeoDataPlacemark *placemark = static_cast<GeoDataPlacemark*>(m_document->child(m_document->size()-1));
-        //GeoDataMultiGeometry *geometry = static_cast<GeoDataMultiGeometry*>(placemark->geometry());
-        m_currentLineString->append(position);
+
+        if ( accuracy.horizontal < 250 ) {
+            m_currentLineString->append(position);
+        }
 
         //if the position has moved then update the current position
-        if ( !( m_gpsCurrentPosition ==
-                position ) )
+        if ( m_gpsCurrentPosition != position )
         {
             placemark = static_cast<GeoDataPlacemark*>(m_document->child(0));
             placemark->setCoordinate(position);
@@ -61,16 +62,18 @@ void PositionTrackingPrivate::setStatus( PositionProviderStatus status )
 {
     if (status == PositionProviderStatusAvailable) {
         Q_ASSERT(m_document);
+        m_treeModel->removeDocument( m_document );
         GeoDataPlacemark *placemark = static_cast<GeoDataPlacemark*>(m_document->child(m_document->size()-1));
         GeoDataMultiGeometry *multiGeometry = static_cast<GeoDataMultiGeometry*>(placemark->geometry());
         m_currentLineString = new GeoDataLineString;
         multiGeometry->append(m_currentLineString);
+        m_treeModel->addDocument( m_document );
     }
 
     emit statusChanged( status );
 }
 
-PositionTracking::PositionTracking( MarbleModel *model )
+PositionTracking::PositionTracking( GeoDataTreeModel *model )
      : QObject( model ), d (new PositionTrackingPrivate( model ))
 {
 
@@ -112,12 +115,11 @@ PositionTracking::PositionTracking( MarbleModel *model )
     styleMap.insert("normal", QString("#").append(style.styleId()));
     d->m_document->addStyleMap(styleMap);
     d->m_document->addStyle(style);
-
-    placemark->setStyleUrl(QString("#").append(styleMap.styleId()));
-    placemark->setStyle( &d->m_document->style(style.styleId()));
     d->m_document->append(placemark);
 
-    d->m_marbleModel->treeModel()->addDocument(d->m_document);
+    placemark->setStyleUrl(QString("#").append(styleMap.styleId()));
+
+    d->m_treeModel->addDocument(d->m_document);
 }
 
 
@@ -172,12 +174,16 @@ qreal PositionTracking::direction() const
 
 bool PositionTracking::trackVisible() const
 {
-    return d->m_document->isVisible();
+    GeoDataPlacemark *placemark = static_cast<GeoDataPlacemark*>(d->m_document->child(d->m_document->size()-1));
+    return placemark->isVisible();
 }
 
 void PositionTracking::setTrackVisible( bool visible )
 {
-    d->m_document->setVisible( visible );
+    d->m_treeModel->removeDocument(d->m_document);
+    GeoDataPlacemark *placemark = static_cast<GeoDataPlacemark*>(d->m_document->child(d->m_document->size()-1));
+    placemark->setVisible( visible );
+    d->m_treeModel->addDocument(d->m_document);
 }
 
 bool PositionTracking::saveTrack(QString& fileName)
@@ -222,9 +228,10 @@ void PositionTracking::clearTrack()
     GeoDataPlacemark *placemark = static_cast<GeoDataPlacemark*>(d->m_document->child(d->m_document->size()-1));
     GeoDataMultiGeometry *multiGeometry = static_cast<GeoDataMultiGeometry*>(placemark->geometry());
     d->m_currentLineString = new GeoDataLineString;
+    d->m_treeModel->removeDocument( d->m_document );
     multiGeometry->clear();
     multiGeometry->append(d->m_currentLineString);
-    d->m_marbleModel->treeModel()->update();
+    d->m_treeModel->addDocument( d->m_document );
 }
 
 bool PositionTracking::isTrackEmpty() const
