@@ -270,7 +270,7 @@ void QtMarbleConfigDialog::readSettings()
     syncSettings();
     
     // View
-    d->ui_viewSettings.kcfg_distanceUnit->setCurrentIndex( distanceUnit() );
+    d->ui_viewSettings.kcfg_distanceUnit->setCurrentIndex( measurementSystem() );
     d->ui_viewSettings.kcfg_angleUnit->setCurrentIndex( angleUnit() );
     d->ui_viewSettings.kcfg_stillQuality->setCurrentIndex( stillQuality() );
     d->ui_viewSettings.kcfg_animationQuality->setCurrentIndex( animationQuality() );
@@ -510,21 +510,15 @@ void QtMarbleConfigDialog::applyPluginState()
     }
 }
 
-Marble::DistanceUnit QtMarbleConfigDialog::distanceUnit() const
+QLocale::MeasurementSystem QtMarbleConfigDialog::measurementSystem() const
 {
     if( d->m_settings->contains( "View/distanceUnit" ) ) {
-        return (Marble::DistanceUnit) d->m_settings->value( "View/distanceUnit" ).toInt();
-    }
-    else {
-        MarbleLocale *locale = MarbleGlobal::getInstance()->locale();
-        
-        if ( locale->measureSystem() == Marble::Metric ) {
-            return Marble::Meter;
-        }
-        else {
-            return Marble::MilesFeet;
-        }
+        return (QLocale::MeasurementSystem) d->m_settings->value( "View/distanceUnit" ).toInt();
     }
+
+    MarbleLocale *locale = MarbleGlobal::getInstance()->locale();
+
+    return locale->measurementSystem();
 }
 
 Marble::AngleUnit QtMarbleConfigDialog::angleUnit() const
@@ -556,12 +550,12 @@ QFont QtMarbleConfigDialog::mapFont() const
 
 Marble::GraphicsSystem QtMarbleConfigDialog::graphicsSystem() const
 {
-    QString graphicsSystemString = d->m_settings->value( "View/graphicsSystem", "native" ).toString();
+    QString graphicsSystemString = d->m_settings->value( "View/graphicsSystem", "raster" ).toString();
 
     if ( graphicsSystemString == "raster" ) return Marble::RasterGraphics;
     if ( graphicsSystemString == "opengl" ) return Marble::OpenGLGraphics;
 
-    // default case:  graphicsSystemString == "native"
+    // default case:  graphicsSystemString == "raster"
     return Marble::NativeGraphics;
 }
 
@@ -589,7 +583,7 @@ bool QtMarbleConfigDialog::animateTargetVoyage() const
 
 int QtMarbleConfigDialog::volatileTileCacheLimit() const
 {
-    int defaultValue = MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen ? 6 : 30;
+    int defaultValue = MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen ? 6 : 100;
     return d->m_settings->value( "Cache/volatileTileCacheLimit", defaultValue ).toInt();
 }
 
