@@ -41,19 +41,17 @@ class RoutingManagerPrivate
 public:
     RoutingManager* q;
 
-    RouteRequest *m_routeRequest;
+    RouteRequest m_routeRequest;
 
-    RoutingModel *m_routingModel;
+    RoutingModel m_routingModel;
 
-    RoutingProfilesModel *m_profilesModel;
+    RoutingProfilesModel m_profilesModel;
 
-    MarbleModel *m_marbleModel;
+    MarbleModel *const m_marbleModel;
 
-    AlternativeRoutesModel* m_alternativeRoutesModel;
+    AlternativeRoutesModel m_alternativeRoutesModel;
 
-    bool m_workOffline;
-
-    MarbleRunnerManager* m_runnerManager;
+    MarbleRunnerManager m_runnerManager;
 
     bool m_haveRoute;
 
@@ -67,6 +65,10 @@ public:
 
     bool m_guidanceModeWarning;
 
+    QString m_lastOpenPath;
+
+    QString m_lastSavePath;
+
     RoutingManagerPrivate( MarbleModel *marbleModel, RoutingManager* manager, QObject *parent );
 
     GeoDataFolder* routeRequest() const;
@@ -76,33 +78,37 @@ public:
     void saveRoute( const QString &filename );
 
     void loadRoute( const QString &filename );
+
+    void addRoute( GeoDataDocument* route );
+
+    void recalculateRoute( bool deviated );
 };
 
 RoutingManagerPrivate::RoutingManagerPrivate( MarbleModel *model, RoutingManager* manager, QObject *parent ) :
         q( manager ),
-        m_routeRequest( new RouteRequest( manager ) ),
-        m_routingModel( new RoutingModel( m_routeRequest, model ) ),
-        m_profilesModel( new RoutingProfilesModel( model->pluginManager() ) ),
+        m_routeRequest( manager ),
+        m_routingModel( &m_routeRequest, model, manager ),
+        m_profilesModel( model->pluginManager() ),
         m_marbleModel( model ),
-        m_alternativeRoutesModel(new AlternativeRoutesModel( model, parent ) ),
-        m_workOffline( false ),
-        m_runnerManager( new MarbleRunnerManager( model->pluginManager(), q ) ),
-        m_haveRoute( false ), m_adjustNavigation( 0 ),
+        m_alternativeRoutesModel( parent ),
+        m_runnerManager( model->pluginManager(), q ),
+        m_haveRoute( false ),
+        m_adjustNavigation( 0 ),
         m_guidanceModeEnabled( false ),
         m_shutdownPositionTracking( false ),
         m_guidanceModeWarning( true )
 {
-    // nothing to do
+    m_runnerManager.setModel( model );
 }
 
 GeoDataFolder* RoutingManagerPrivate::routeRequest() const
 {
     GeoDataFolder* result = new GeoDataFolder;
     result->setName( "Route Request" );
-    for ( int i=0; i<m_routeRequest->size(); ++i ) {
+    for ( int i=0; i<m_routeRequest.size(); ++i ) {
         GeoDataPlacemark* placemark = new GeoDataPlacemark;
-        placemark->setName( m_routeRequest->name( i ) );
-        placemark->setCoordinate( GeoDataPoint( m_routeRequest->at( i ) ) );
+        placemark->setName( m_routeRequest.name( i ) );
+        placemark->setCoordinate( GeoDataPoint( m_routeRequest.at( i ) ) );
         result->append( placemark );
     }
 
@@ -146,7 +152,7 @@ void RoutingManagerPrivate::saveRoute(const QString &filename)
         container.append( request );
     }
 
-    GeoDataDocument *route = m_alternativeRoutesModel->currentRoute();
+    GeoDataDocument *route = m_alternativeRoutesModel.currentRoute();
     if ( route ) {
         container.append( new GeoDataDocument( *route ) );
     }
@@ -172,44 +178,46 @@ void RoutingManagerPrivate::loadRoute(const QString &filename)
     }
 
     GeoDocument *doc = parser.releaseDocument();
+    file.close();
+    bool loaded = false;
+
     GeoDataDocument* container = dynamic_cast<GeoDataDocument*>( doc );
-    if ( container && container->size() == 2 ) {
+    if ( container && container->size() > 0 ) {
         GeoDataFolder* viaPoints = dynamic_cast<GeoDataFolder*>( &container->first() );
         if ( viaPoints ) {
+            loaded = true;
             QVector<GeoDataPlacemark*> placemarks = viaPoints->placemarkList();
             for( int i=0; i<placemarks.size(); ++i ) {
-                if ( i < m_routeRequest->size() ) {
-                    m_routeRequest->setPosition( i, placemarks[i]->coordinate() );
+                if ( i < m_routeRequest.size() ) {
+                    m_routeRequest.setPosition( i, placemarks[i]->coordinate() );
                 } else {
-                    m_routeRequest->append( placemarks[i]->coordinate() );
+                    m_routeRequest.append( placemarks[i]->coordinate() );
                 }
-                m_routeRequest->setName( m_routeRequest->size()-1, placemarks[i]->name() );
+                m_routeRequest.setName( m_routeRequest.size()-1, placemarks[i]->name() );
             }
 
-            for ( int i=placemarks.size(); i<m_routeRequest->size(); ++i ) {
-                m_routeRequest->remove( i );
+            for ( int i=placemarks.size(); i<m_routeRequest.size(); ++i ) {
+                m_routeRequest.remove( i );
             }
         } else {
-            mDebug() << "Expected a GeoDataDocument with two children, didn't get one though";
+            mDebug() << "Expected a GeoDataDocument with at least one child, didn't get one though";
         }
-
-        GeoDataDocument* route = dynamic_cast<GeoDataDocument*>(&container->last());
-        if ( route ) {
-            m_alternativeRoutesModel->addRoute( route, AlternativeRoutesModel::Instant );
-            m_alternativeRoutesModel->setCurrentRoute( 0 );
-        } else {
-            mDebug() << "Expected a GeoDataDocument child, didn't get one though";
-            delete doc;
-            doc = 0;
-        }
-    } else {
-        mDebug() << "Expected a GeoDataDocument, didn't get one though";
-        delete doc;
-        doc = 0;
     }
 
-    file.close();
-    if ( !doc ) {
+    if ( container && container->size() == 2 ) {
+        GeoDataDocument* route = dynamic_cast<GeoDataDocument*>(&container->last());
+        if ( route ) {
+            loaded = true;
+            m_alternativeRoutesModel.addRoute( route, AlternativeRoutesModel::Instant );
+            m_alternativeRoutesModel.setCurrentRoute( 0 );
+        } else {
+            mDebug() << "Expected a GeoDataDocument child, didn't get one though";
+        }
+    }
+
+    if ( !loaded ) {
+        mDebug() << "File " << filename << " is not a valid Marble route .kml file";
+        delete doc;
         m_marbleModel->addGeoDataFile( filename );
     }
 }
@@ -217,11 +225,11 @@ void RoutingManagerPrivate::loadRoute(const QString &filename)
 RoutingManager::RoutingManager( MarbleModel *marbleModel, QObject *parent ) : QObject( parent ),
         d( new RoutingManagerPrivate( marbleModel, this, this ) )
 {
-    connect( d->m_runnerManager, SIGNAL( routeRetrieved( GeoDataDocument* ) ),
-             this, SLOT( retrieveRoute( GeoDataDocument* ) ) );
-    connect( d->m_alternativeRoutesModel, SIGNAL( currentRouteChanged( GeoDataDocument* ) ),
-             d->m_routingModel, SLOT( setCurrentRoute( GeoDataDocument* ) ) );
-    connect( d->m_routingModel, SIGNAL( deviatedFromRoute( bool ) ),
+    connect( &d->m_runnerManager, SIGNAL( routeRetrieved( GeoDataDocument* ) ),
+             this, SLOT( addRoute( GeoDataDocument* ) ) );
+    connect( &d->m_alternativeRoutesModel, SIGNAL( currentRouteChanged( GeoDataDocument* ) ),
+             &d->m_routingModel, SLOT( setCurrentRoute( GeoDataDocument* ) ) );
+    connect( &d->m_routingModel, SIGNAL( deviatedFromRoute( bool ) ),
              this, SLOT( recalculateRoute( bool ) ) );
 }
 
@@ -232,74 +240,58 @@ RoutingManager::~RoutingManager()
 
 RoutingProfilesModel *RoutingManager::profilesModel()
 {
-    return d->m_profilesModel;
+    return &d->m_profilesModel;
 }
 
 RoutingModel *RoutingManager::routingModel()
 {
-    return d->m_routingModel;
-}
-
-void RoutingManager::retrieveRoute( RouteRequest *route )
-{
-    d->m_routeRequest = route;
-    updateRoute();
+    return &d->m_routingModel;
 }
 
 RouteRequest* RoutingManager::routeRequest()
 {
-    return d->m_routeRequest;
+    return &d->m_routeRequest;
 }
 
 void RoutingManager::updateRoute()
 {
-    if ( !d->m_routeRequest ) {
-        return;
-    }
-
     d->m_haveRoute = false;
 
     int realSize = 0;
-    for ( int i = 0; i < d->m_routeRequest->size(); ++i ) {
+    for ( int i = 0; i < d->m_routeRequest.size(); ++i ) {
         // Sort out dummy targets
-        if ( d->m_routeRequest->at( i ).longitude() != 0.0 && d->m_routeRequest->at( i ).latitude() != 0.0 ) {
+        if ( d->m_routeRequest.at( i ).longitude() != 0.0 && d->m_routeRequest.at( i ).latitude() != 0.0 ) {
             ++realSize;
         }
     }
 
-    d->m_alternativeRoutesModel->newRequest( d->m_routeRequest );
+    d->m_alternativeRoutesModel.newRequest( &d->m_routeRequest );
     if ( realSize > 1 ) {
-        emit stateChanged( RoutingManager::Downloading, d->m_routeRequest );
-        d->m_runnerManager->setWorkOffline( d->m_workOffline );
-        d->m_runnerManager->retrieveRoute( d->m_routeRequest );
+        emit stateChanged( RoutingManager::Downloading, &d->m_routeRequest );
+        d->m_runnerManager.retrieveRoute( &d->m_routeRequest );
     } else {
-        d->m_routingModel->clear();
-        emit stateChanged( RoutingManager::Retrieved, d->m_routeRequest );
+        d->m_routingModel.clear();
+        emit stateChanged( RoutingManager::Retrieved, &d->m_routeRequest );
     }
 }
 
-void RoutingManager::setWorkOffline( bool offline )
-{
-    d->m_workOffline = offline;
-}
-
-void RoutingManager::retrieveRoute( GeoDataDocument* route )
+void RoutingManagerPrivate::addRoute( GeoDataDocument* route )
 {
     if ( route ) {
-        d->m_alternativeRoutesModel->addRoute( route );
+        m_alternativeRoutesModel.addRoute( route );
     }
 
-    if ( !d->m_haveRoute ) {
-        d->m_haveRoute = route != 0;
-        emit stateChanged( Retrieved, d->m_routeRequest );
+    if ( !m_haveRoute ) {
+        m_haveRoute = route != 0;
+        emit q->stateChanged( RoutingManager::Retrieved, &m_routeRequest );
     }
 
-    emit routeRetrieved( route );
+    emit q->routeRetrieved( route );
 }
 
 AlternativeRoutesModel* RoutingManager::alternativeRoutesModel()
 {
-    return d->m_alternativeRoutesModel;
+    return &d->m_alternativeRoutesModel;
 }
 
 void RoutingManager::setAdjustNavigation( AdjustNavigation* adjustNavigation )
@@ -307,7 +299,7 @@ void RoutingManager::setAdjustNavigation( AdjustNavigation* adjustNavigation )
     d->m_adjustNavigation = adjustNavigation;
 }
 
-AdjustNavigation* RoutingManager::adjustNavigation()
+const AdjustNavigation* RoutingManager::adjustNavigation() const
 {
     return d->m_adjustNavigation;
 }
@@ -330,8 +322,8 @@ void RoutingManager::loadRoute( const QString &filename )
 void RoutingManager::readSettings()
 {
     d->loadRoute( d->stateFile() );
-    if ( d->m_routeRequest && d->m_profilesModel->rowCount() ) {
-        d->m_routeRequest->setRoutingProfile( d->m_profilesModel->profiles().at( 0 ) );
+    if ( d->m_profilesModel.rowCount() ) {
+        d->m_routeRequest.setRoutingProfile( d->m_profilesModel.profiles().at( 0 ) );
     }
 }
 
@@ -364,7 +356,7 @@ void RoutingManager::setGuidanceModeEnabled( bool enabled )
     PositionTracking* tracking = d->m_marbleModel->positionTracking();
     PositionProviderPlugin* plugin = tracking->positionProviderPlugin();
     if ( !plugin && enabled ) {
-        PluginManager* pluginManager = d->m_marbleModel->pluginManager();
+        const PluginManager* pluginManager = d->m_marbleModel->pluginManager();
         QList<PositionProviderPlugin*> plugins = pluginManager->createPositionProviderPlugins();
         if ( plugins.size() > 0 ) {
             plugin = plugins.takeFirst();
@@ -377,38 +369,38 @@ void RoutingManager::setGuidanceModeEnabled( bool enabled )
         tracking->setPositionProviderPlugin( 0 );
     }
 
-    adjustNavigation()->setAutoZoom( enabled );
-    adjustNavigation()->setRecenter( enabled ? AdjustNavigation::RecenterOnBorder : AdjustNavigation::DontRecenter );
+    d->m_adjustNavigation->setAutoZoom( enabled );
+    d->m_adjustNavigation->setRecenter( enabled ? AdjustNavigation::RecenterOnBorder : AdjustNavigation::DontRecenter );
 }
 
-void RoutingManager::recalculateRoute( bool deviated )
+void RoutingManagerPrivate::recalculateRoute( bool deviated )
 {
-    if ( d->m_guidanceModeEnabled && deviated && d->m_routeRequest ) {
-        for ( int i=d->m_routeRequest->size()-3; i>=0; --i ) {
-            if ( d->m_routeRequest->visited( i ) ) {
-                d->m_routeRequest->remove( i );
+    if ( m_guidanceModeEnabled && deviated ) {
+        for ( int i=m_routeRequest.size()-3; i>=0; --i ) {
+            if ( m_routeRequest.visited( i ) ) {
+                m_routeRequest.remove( i );
             }
         }
 
-        if ( d->m_routeRequest->size() == 2 && d->m_routeRequest->visited( 0 ) && !d->m_routeRequest->visited( 1 ) ) {
-            d->m_routeRequest->setPosition( 0, d->m_marbleModel->positionTracking()->currentLocation() );
-            updateRoute();
-        } else if ( d->m_routeRequest->size() != 0 && !d->m_routeRequest->visited( d->m_routeRequest->size()-1 ) ) {
-            d->m_routeRequest->insert( 0, d->m_marbleModel->positionTracking()->currentLocation() );
-            updateRoute();
+        if ( m_routeRequest.size() == 2 && m_routeRequest.visited( 0 ) && !m_routeRequest.visited( 1 ) ) {
+            m_routeRequest.setPosition( 0, m_marbleModel->positionTracking()->currentLocation() );
+            q->updateRoute();
+        } else if ( m_routeRequest.size() != 0 && !m_routeRequest.visited( m_routeRequest.size()-1 ) ) {
+            m_routeRequest.insert( 0, m_marbleModel->positionTracking()->currentLocation() );
+            q->updateRoute();
         }
     }
 }
 
 void RoutingManager::reverseRoute()
 {
-    d->m_routeRequest->reverse();
+    d->m_routeRequest.reverse();
     updateRoute();
 }
 
 void RoutingManager::clearRoute()
 {
-    d->m_routeRequest->clear();
+    d->m_routeRequest.clear();
     updateRoute();
 }
 
@@ -422,6 +414,26 @@ bool RoutingManager::showGuidanceModeStartupWarning() const
     return d->m_guidanceModeWarning;
 }
 
+void RoutingManager::setLastOpenPath( const QString &path )
+{
+    d->m_lastOpenPath = path;
+}
+
+QString RoutingManager::lastOpenPath() const
+{
+    return d->m_lastOpenPath;
+}
+
+void RoutingManager::setLastSavePath( const QString &path )
+{
+    d->m_lastSavePath = path;
+}
+
+QString RoutingManager::lastSavePath() const
+{
+    return d->m_lastSavePath;
+}
+
 } // namespace Marble
 
 #include "RoutingManager.moc"
