@@ -80,7 +80,6 @@
 #include "routing/RouteRequest.h"
 #include "SunControlWidget.h"
 #include "TimeControlWidget.h"
-#include "SunLocator.h"
 #include "TileCoordsPyramid.h"
 #include "ViewportParams.h"
 #include "MarbleClock.h"
@@ -135,10 +134,10 @@ MarblePart::MarblePart( QWidget *parentWidget, QObject *parent, const QVariantLi
     MarbleLocale *marbleLocale = MarbleGlobal::getInstance()->locale();
     KLocale *kLocale = KGlobal::locale();
     if ( kLocale->measureSystem() == KLocale::Metric ) {
-        marbleLocale->setMeasureSystem( Metric );
+        marbleLocale->setMeasurementSystem( QLocale::MetricSystem );
     }
     else {
-        marbleLocale->setMeasureSystem( Imperial );
+        marbleLocale->setMeasurementSystem( QLocale::ImperialSystem );
     }
 
     m_externalEditorMapping[0] = "";
@@ -214,10 +213,20 @@ bool MarblePart::openUrl( const KUrl &url )
 
 bool MarblePart::openFile()
 {
-    QStringList fileNames = KFileDialog::getOpenFileNames( KUrl(),
-                                    i18n("*.gpx *.kml|All Supported Files\n*.gpx|GPS Data\n*.kml|Google Earth KML"),
+    QStringList fileNames = KFileDialog::getOpenFileNames( m_lastFileOpenPath,
+                                    i18n("*.gpx *.kml *.osm *.pnt|All Supported Files\n"
+                                         "*.gpx|GPS Data\n"
+                                         "*.kml|Google Earth KML\n"
+                                         "*.osm|OpenStreetMap Data\n"
+                                         "*.pnt|Micro World Data Bank II"),
                                             widget(), i18n("Open File")
                                            );
+
+    if ( !fileNames.isEmpty() ) {
+        const QString firstFile = fileNames.first();
+        m_lastFileOpenPath = KUrl::fromLocalFile( QFileInfo( firstFile ).absolutePath() );
+    }
+
     foreach( const QString &fileName, fileNames ) {
         m_controlView->marbleModel()->addGeoDataFile( fileName );
     }
@@ -320,7 +329,7 @@ void MarblePart::showStatusBar( bool isChecked )
 void MarblePart::controlSun()
 {
     if ( !m_sunControlDialog ) {
-        m_sunControlDialog = new SunControlWidget( m_controlView->sunLocator() );
+        m_sunControlDialog = new SunControlWidget( m_controlView->marbleWidget(), m_controlView );
         connect( m_sunControlDialog, SIGNAL( showSun( bool ) ),
                  this,               SLOT ( showSun( bool ) ) );
         connect( m_sunControlDialog, SIGNAL( showSun( bool ) ),
@@ -336,7 +345,7 @@ void MarblePart::controlSun()
 
 void MarblePart::controlTime()
 {
-    if ( !m_timeControlDialog ) 
+    if ( !m_timeControlDialog )
     {
         m_timeControlDialog = new TimeControlWidget( m_controlView->marbleModel()->clock() );
     }
@@ -348,13 +357,13 @@ void MarblePart::controlTime()
 
 void MarblePart::showSun( bool active )
 {
-    m_controlView->sunLocator()->setShow( active ); 
+    m_controlView->marbleWidget()->setShowSunShading( active );
     m_sunControlDialog->setSunShading( active );
 }
 
 void MarblePart::showSunInZenith( bool active )
 {
-    m_controlView->sunLocator()->setCentered( active );
+    m_controlView->marbleWidget()->setShowSunInZenith( active );
 }
 
 void MarblePart::workOffline( bool offline )
@@ -417,18 +426,20 @@ void MarblePart::readSettings()
     m_showAtmosphereAction->setChecked( MarbleSettings::showAtmosphere() );
     m_lockFloatItemsAct->setChecked(MarbleSettings::lockFloatItemPositions());
     lockFloatItemPosition(MarbleSettings::lockFloatItemPositions());
-    
+
     // Sun
-    m_controlView->sunLocator()->setShow( MarbleSettings::showSun() );
+    m_controlView->marbleWidget()->setShowSunShading( MarbleSettings::showSun() );
     m_showShadow->setChecked( MarbleSettings::showSun() );
-    m_controlView->sunLocator()->setCitylights( MarbleSettings::showCitylights() );
-    m_controlView->sunLocator()->setCentered( MarbleSettings::centerOnSun() );
+    m_controlView->marbleWidget()->setShowCityLights( MarbleSettings::showCitylights() );
+    m_controlView->marbleWidget()->setShowSunInZenith( MarbleSettings::centerOnSun() );
     m_showSunInZenith->setChecked( MarbleSettings::centerOnSun() );
 
     // View
     m_initialGraphicsSystem = (GraphicsSystem) MarbleSettings::graphicsSystem();
     m_previousGraphicsSystem = m_initialGraphicsSystem;
 
+    m_lastFileOpenPath = KUrl::fromLocalFile( MarbleSettings::lastFileOpenDir() );
+
     // Plugins
     QHash<QString, int> pluginEnabled;
     QHash<QString, int> pluginVisible;
@@ -443,7 +454,7 @@ void MarblePart::readSettings()
                 = MarbleSettings::pluginEnabled()[i];
         }
     }
-    
+
     if ( nameIdSize == visibleSize ) {
         for ( int i = 0; i < visibleSize; ++i ) {
             pluginVisible[ MarbleSettings::pluginNameId()[i] ]
@@ -503,11 +514,12 @@ void MarblePart::readSettings()
     QString positionProvider = MarbleSettings::activePositionTrackingPlugin();
     if ( !positionProvider.isEmpty() ) {
         PositionTracking* tracking = m_controlView->marbleModel()->positionTracking();
-        PluginManager* pluginManager = m_controlView->marbleModel()->pluginManager();
+        const PluginManager* pluginManager = m_controlView->marbleModel()->pluginManager();
         QList<PositionProviderPlugin*> plugins = pluginManager->createPositionProviderPlugins();
         foreach( PositionProviderPlugin* plugin, plugins ) {
             if ( plugin->nameId() == positionProvider ) {
                 plugins.removeOne( plugin );
+                plugin->setMarbleModel( m_controlView->marbleModel() );
                 tracking->setPositionProviderPlugin( plugin );
                 break;
             }
@@ -522,8 +534,7 @@ void MarblePart::readSettings()
     // Time
     if( MarbleSettings::systemTime() == true  )
     {
-        m_controlView->marbleModel()->setClockDateTime( QDateTime::currentDateTime().toUTC() );
-        m_controlView->marbleModel()->setClockSpeed( 1 );
+        /* nothing to do */
     }
     else if( MarbleSettings::lastSessionTime() == true )
     {
@@ -612,21 +623,22 @@ void MarblePart::writeSettings()
     GraphicsSystem graphicsSystem = (GraphicsSystem) MarbleSettings::graphicsSystem();
     MarbleSettings::setGraphicsSystem( graphicsSystem );
 
+    MarbleSettings::setLastFileOpenDir( m_lastFileOpenPath.toLocalFile() );
 
-    MarbleSettings::setDistanceUnit( MarbleGlobal::getInstance()->locale()->distanceUnit() );
+    MarbleSettings::setDistanceUnit( MarbleGlobal::getInstance()->locale()->measurementSystem() );
     MarbleSettings::setAngleUnit( m_controlView->marbleWidget()->defaultAngleUnit() );
 
     // Sun
-    MarbleSettings::setShowSun( m_controlView->sunLocator()->getShow() );
-    MarbleSettings::setShowCitylights( m_controlView->sunLocator()->getCitylights() );
-    MarbleSettings::setCenterOnSun( m_controlView->sunLocator()->getCentered() );
+    MarbleSettings::setShowSun( m_controlView->marbleWidget()->showSunShading() );
+    MarbleSettings::setShowCitylights( m_controlView->marbleWidget()->showCityLights() );
+    MarbleSettings::setCenterOnSun( m_controlView->marbleWidget()->showSunInZenith() );
 
     // Caches
     MarbleSettings::setVolatileTileCacheLimit( m_controlView->marbleWidget()->
                                                volatileTileCacheLimit() / 1024 );
     MarbleSettings::setPersistentTileCacheLimit( m_controlView->marbleModel()->
                                                  persistentTileCacheLimit() / 1024 );
-    
+
     // Time
     MarbleSettings::setDateTime( m_controlView->marbleModel()->clockDateTime() );
     MarbleSettings::setSpeedSlider( m_controlView->marbleModel()->clockSpeed() );
@@ -640,9 +652,9 @@ void MarblePart::writeSettings()
     QList<RenderPlugin *>::const_iterator i = pluginList.constBegin();
     QList<RenderPlugin *>::const_iterator const end = pluginList.constEnd();
     for (; i != end; ++i ) {
-	pluginEnabled << static_cast<int>( (*i)->enabled() );
+        pluginEnabled << static_cast<int>( (*i)->enabled() );
         pluginVisible << static_cast<int>( (*i)->visible() );
-	pluginNameId  << (*i)->nameId();
+        pluginNameId  << (*i)->nameId();
     }
     MarbleSettings::setPluginEnabled( pluginEnabled );
     MarbleSettings::setPluginVisible( pluginVisible );
@@ -671,6 +683,8 @@ void MarblePart::writeSettings()
     QList<QString> const editors = m_externalEditorMapping.values();
     MarbleSettings::setExternalMapEditor( editors.indexOf( m_controlView->externalMapEditor() ) );
 
+    applyPluginState();
+
     MarbleSettings::self()->writeConfig();
 }
 
@@ -730,26 +744,26 @@ void MarblePart::setupActions()
 
     // Action: Copy Map to the Clipboard
     m_copyMapAction = KStandardAction::copy( this, SLOT( copyMap() ),
-					     actionCollection() );
+                                             actionCollection() );
     m_copyMapAction->setText( i18nc( "Action for copying the map to the clipboard", "&Copy Map" ) );
 
     // Action: Copy Coordinates string
     m_copyCoordinatesAction = new KAction( this );
     actionCollection()->addAction( "edit_copy_coordinates",
-				   m_copyCoordinatesAction );
+                                   m_copyCoordinatesAction );
     m_copyCoordinatesAction->setText( i18nc( "Action for copying the coordinates to the clipboard",
                                              "C&opy Coordinates" ) );
     connect( m_copyCoordinatesAction, SIGNAL( triggered( bool ) ),
-	     this,                    SLOT( copyCoordinates() ) );
+             this,                    SLOT( copyCoordinates() ) );
 
     // Action: Open a Gpx or a Kml File
     m_openAct = KStandardAction::open( this, SLOT( openFile() ),
-				       actionCollection() );
+                                       actionCollection() );
     m_openAct->setText( i18nc( "Action for opening a file", "&Open..." ) );
 
     // Standard actions.  So far only Quit.
     KStandardAction::quit( kapp, SLOT( closeAllWindows() ),
-			   actionCollection() );
+                           actionCollection() );
 
     // Action: Get hot new stuff
     m_newStuffAction = KNS3::standardAction( i18nc( "Action for downloading maps (GHNS)",
@@ -770,7 +784,7 @@ void MarblePart::setupActions()
     connect( m_mapWizardAct, SIGNAL( triggered() ), SLOT( showMapWizard() ) );
 
     KStandardAction::showStatusbar( this, SLOT( showStatusBar( bool ) ),
-				    actionCollection() );
+                                    actionCollection() );
 
     m_sideBarAct = new KAction( i18nc( "Action for toggling the navigation panel",
                                        "Show &Navigation Panel"), this );
@@ -780,12 +794,12 @@ void MarblePart::setupActions()
     m_sideBarAct->setChecked( true );
     m_sideBarAct->setStatusTip( i18nc( "Status tip", "Show Navigation Panel" ) );
     connect( m_sideBarAct, SIGNAL( triggered( bool ) ),
-	     this,         SLOT( showSideBar( bool ) ) );
+             this,         SLOT( showSideBar( bool ) ) );
 
     m_fullScreenAct = KStandardAction::fullScreen( 0, 0, widget(),
-						   actionCollection() );
+                                                   actionCollection() );
     connect( m_fullScreenAct, SIGNAL( triggered( bool ) ),
-	     this,            SLOT( showFullScreen( bool ) ) );
+             this,            SLOT( showFullScreen( bool ) ) );
 
     // Action: Show Atmosphere option
     m_showAtmosphereAction = new KAction( this );
@@ -794,7 +808,7 @@ void MarblePart::setupActions()
     m_showAtmosphereAction->setChecked( true );
     m_showAtmosphereAction->setText( i18nc( "Action for toggling the atmosphere", "&Atmosphere" ) );
     connect( m_showAtmosphereAction, SIGNAL( triggered( bool ) ),
-	     this,                   SLOT( setShowAtmosphere( bool ) ) );
+             this,                   SLOT( setShowAtmosphere( bool ) ) );
 
     // Action: Show Crosshairs option
     QList<RenderPlugin *> pluginList = m_controlView->marbleWidget()->renderPlugins();
@@ -814,14 +828,14 @@ void MarblePart::setupActions()
     m_showCloudsAction->setChecked( true );
     m_showCloudsAction->setText( i18nc( "Action for toggling clouds", "&Clouds" ) );
     connect( m_showCloudsAction, SIGNAL( triggered( bool ) ),
-	     this,               SLOT( setShowClouds( bool ) ) );
+             this,               SLOT( setShowClouds( bool ) ) );
 
     // Action: Show Sunshade options
     m_controlSunAction = new KAction( this );
     actionCollection()->addAction( "control_sun", m_controlSunAction );
     m_controlSunAction->setText( i18nc( "Action for sun control dialog", "S&un Control..." ) );
     connect( m_controlSunAction, SIGNAL( triggered( bool ) ),
-	     this,               SLOT( controlSun() ) );
+             this,               SLOT( controlSun() ) );
 
     KStandardAction::redisplay( m_controlView->marbleWidget(), SLOT( reloadMap() ), actionCollection() );
 
@@ -835,16 +849,16 @@ void MarblePart::setupActions()
     // Action: Lock float items
     m_lockFloatItemsAct = new KAction ( this );
     actionCollection()->addAction( "options_lock_floatitems",
-				   m_lockFloatItemsAct );
+                                   m_lockFloatItemsAct );
     m_lockFloatItemsAct->setText( i18nc( "Action for locking float items on the map",
                                          "Lock Position" ) );
     m_lockFloatItemsAct->setCheckable( true );
     m_lockFloatItemsAct->setChecked( false );
     connect( m_lockFloatItemsAct, SIGNAL( triggered( bool ) ),
-	     this,                SLOT( lockFloatItemPosition( bool ) ) );
+             this,                SLOT( lockFloatItemPosition( bool ) ) );
 
     KStandardAction::preferences( this, SLOT( editSettings() ),
-				  actionCollection() );
+                                  actionCollection() );
 
     //Toggle Action: Show sun shadow
     m_showShadow = new KToggleAction( i18n( "Show Shadow" ), this );
@@ -861,7 +875,7 @@ void MarblePart::setupActions()
     m_showSunInZenith->setCheckedState( KGuiItem( i18n( "Hide Zenith" ) ) );
     m_showSunInZenith->setToolTip( i18n( "Shows and hides the zenith location of the sun" ) );
     connect( m_showSunInZenith, SIGNAL( triggered( bool ) ), this, SLOT( showSunInZenith( bool ) ));
-    
+
     //    FIXME: Discuss if this is the best place to put this
     QList<RenderPlugin *>::const_iterator it = pluginList.constBegin();
     QList<RenderPlugin *>::const_iterator const itEnd = pluginList.constEnd();
@@ -911,12 +925,12 @@ void MarblePart::createFolderList()
 
    for (; i != end; ++i ) {
        QMenu *m_bookmarksListMenu = new QMenu( (*i)->name() );
-   
+
        createBookmarksListMenu( m_bookmarksListMenu, *(*i) );
        connect( m_bookmarksListMenu, SIGNAL( triggered ( QAction *) ),
             this, SLOT( lookAtBookmark( QAction *) ) );
-   
-       actionList.append( m_bookmarksListMenu->menuAction() ); 
+
+       actionList.append( m_bookmarksListMenu->menuAction() );
    }
    unplugActionList("folders");
    plugActionList( "folders", actionList );
@@ -926,10 +940,10 @@ void MarblePart::createBookmarksListMenu( QMenu *m_bookmarksListMenu, const GeoD
 {
     m_bookmarksListMenu->clear();
     QVector<GeoDataPlacemark*> bookmarks = folder.placemarkList();
-    
+
     QVector<GeoDataPlacemark*>::const_iterator i = bookmarks.constBegin();
     QVector<GeoDataPlacemark*>::const_iterator end = bookmarks.constEnd();
-    
+
     for (; i != end; ++i ) {
         QAction *bookmarkAct = new QAction( (*i)->name(), this );
         QVariant var;
@@ -966,19 +980,19 @@ void MarblePart::createInfoBoxesMenu()
 void MarblePart::createOnlineServicesMenu()
 {
     QList<RenderPlugin *> renderPluginList = m_controlView->marbleWidget()->renderPlugins();
-    
+
     QList<QAction*> actionList;
-    
+
     QList<RenderPlugin *>::const_iterator i = renderPluginList.constBegin();
     QList<RenderPlugin *>::const_iterator const end = renderPluginList.constEnd();
     for (; i != end; ++i ) {
         // FIXME: This will go into the layer manager when AbstractDataPlugin is an interface
-       
+
         if( (*i)->renderType() == RenderPlugin::Online ) {
             actionList.append( (*i)->action() );
         }
     }
-    
+
     unplugActionList( "onlineservices_actionlist" );
     plugActionList( "onlineservices_actionlist", actionList );
 }
@@ -1086,7 +1100,7 @@ void MarblePart::setupStatusBar()
              this,                          SLOT( updateStatusBar() ) );
     connect( m_controlView->marbleWidget(), SIGNAL( tileLevelChanged( int )),
              SLOT( showZoomLevel( int )));
-    connect( m_controlView->marbleModel(), SIGNAL( themeChanged( QString )),
+    connect( m_controlView->marbleWidget(), SIGNAL( themeChanged( QString )),
              this, SLOT( mapThemeChanged( QString )), Qt::QueuedConnection );
     connect( m_controlView->marbleModel()->clock(), SIGNAL( timeChanged() ),
              this,                          SLOT( showDateTime() ) );
@@ -1105,7 +1119,7 @@ QLabel * MarblePart::setupStatusBarLabel( const QString& templateString )
     QLabel * const label = new QLabel( m_statusBarExtension->statusBar() );
     label->setIndent( 5 );
     int maxWidth = statusBarFontMetrics.boundingRect( templateString ).width()
-	+ 2 * label->margin() + 2 * label->indent();
+        + 2 * label->margin() + 2 * label->indent();
     label->setFixedWidth( maxWidth );
     m_statusBarExtension->addStatusBarItem( label, -1, false );
     return label;
@@ -1273,7 +1287,7 @@ void MarblePart::editSettings()
         return;
 
     m_configDialog = new KConfigDialog( m_controlView, "settings",
-					MarbleSettings::self() );
+                                        MarbleSettings::self() );
 
     // view page
     Ui_MarbleViewSettingsWidget  ui_viewSettings;
@@ -1305,17 +1319,17 @@ void MarblePart::editSettings()
     w_navigationSettings->setObjectName( "navigation_page" );
     ui_navigationSettings.setupUi( w_navigationSettings );
     m_configDialog->addPage( w_navigationSettings, i18n( "Navigation" ),
-			     "transform-move" );
+                             "transform-move" );
 
     // cache page
     MarbleCacheSettingsWidget *w_cacheSettings = new MarbleCacheSettingsWidget();
     w_cacheSettings->setObjectName( "cache_page" );
     m_configDialog->addPage( w_cacheSettings, i18n( "Cache & Proxy" ),
-			     "preferences-web-browser-cache" );
+                             "preferences-web-browser-cache" );
     connect( w_cacheSettings,               SIGNAL( clearVolatileCache() ),
-	     m_controlView->marbleWidget(), SLOT( clearVolatileTileCache() ) );
+             m_controlView->marbleWidget(), SLOT( clearVolatileTileCache() ) );
     connect( w_cacheSettings,                        SIGNAL( clearPersistentCache() ),
-	     m_controlView->marbleModel(), SLOT( clearPersistentTileCache() ) );
+             m_controlView->marbleModel(), SLOT( clearPersistentTileCache() ) );
 
     // time page
     Ui_MarbleTimeSettingsWidget ui_timeSettings;
@@ -1338,14 +1352,14 @@ void MarblePart::editSettings()
     QList<RenderPlugin *>::const_iterator i = pluginList.constBegin();
     QList<RenderPlugin *>::const_iterator const end = pluginList.constEnd();
     for (; i != end; ++i ) {
-	parentItem->appendRow( (*i)->item() );
+        parentItem->appendRow( (*i)->item() );
     }
 
     MarblePluginSettingsWidget *w_pluginSettings = new MarblePluginSettingsWidget();
     w_pluginSettings->setModel( m_pluginModel );
     w_pluginSettings->setObjectName( "plugin_page" );
     m_configDialog->addPage( w_pluginSettings, i18n( "Plugins" ),
-			     "preferences-plugin" );
+                             "preferences-plugin" );
     // Setting the icons of the pluginSettings page.
     w_pluginSettings->setConfigIcon( KIcon( "configure" ) );
     w_pluginSettings->setAboutIcon( KIcon( "help-about" ) );
@@ -1353,13 +1367,13 @@ void MarblePart::editSettings()
     connect( w_pluginSettings, SIGNAL( pluginListViewClicked() ),
                                SLOT( enableApplyButton() ) );
     connect( m_configDialog,   SIGNAL( settingsChanged( const QString &) ),
-	                       SLOT( updateSettings() ) );
+                               SLOT( updateSettings() ) );
     connect( m_configDialog,   SIGNAL( applyClicked() ),
-	                       SLOT( applyPluginState() ) );
+                               SLOT( applyPluginState() ) );
     connect( m_configDialog,   SIGNAL( okClicked() ),
-	                       SLOT( applyPluginState() ) );
+                               SLOT( applyPluginState() ) );
     connect( m_configDialog,   SIGNAL( cancelClicked() ),
-	                       SLOT( retrievePluginState() ) );
+                               SLOT( retrievePluginState() ) );
     connect( w_pluginSettings, SIGNAL( aboutPluginClicked( QString ) ),
                                SLOT( showPluginAboutDialog( QString ) ) );
     connect( w_pluginSettings, SIGNAL( configPluginClicked( QString ) ),
@@ -1434,7 +1448,7 @@ void MarblePart::updateSettings()
     m_controlView->marbleWidget()->
         setDefaultAngleUnit( (AngleUnit) MarbleSettings::angleUnit() );
     MarbleGlobal::getInstance()->locale()->
-        setDistanceUnit( (DistanceUnit) MarbleSettings::distanceUnit() );
+        setMeasurementSystem( (QLocale::MeasurementSystem) MarbleSettings::distanceUnit() );
 
     updateStatusBar();
 
@@ -1445,14 +1459,11 @@ void MarblePart::updateSettings()
         setPersistentTileCacheLimit( MarbleSettings::persistentTileCacheLimit() * 1024 );
     m_controlView->marbleWidget()->
         setVolatileTileCacheLimit( MarbleSettings::volatileTileCacheLimit() * 1024 );
-/*
-    m_controlView->marbleWidget()->setProxy( MarbleSettings::proxyUrl(),
-                                             MarbleSettings::proxyPort(), MarbleSettings::user(), MarbleSettings::password() );
-*/
+
     //Create and export the proxy
     QNetworkProxy proxy;
-    
-    // Make sure that no proxy is used for an empty string or the default value: 
+
+    // Make sure that no proxy is used for an empty string or the default value:
     if ( MarbleSettings::proxyUrl().isEmpty() || MarbleSettings::proxyUrl() == "http://" ) {
         proxy.setType( QNetworkProxy::NoProxy );
     } else {
@@ -1467,28 +1478,28 @@ void MarblePart::updateSettings()
             proxy.setType( QNetworkProxy::HttpProxy );
         }
     }
-    
+
     proxy.setHostName( MarbleSettings::proxyUrl() );
     proxy.setPort( MarbleSettings::proxyPort() );
-    
+
     if ( MarbleSettings::proxyAuth() ) {
         proxy.setUser( MarbleSettings::proxyUser() );
         proxy.setPassword( MarbleSettings::proxyPass() );
     }
-    
+
     QNetworkProxy::setApplicationProxy(proxy);
-    
+
     m_controlView->marbleWidget()->update();
 
     // Show message box
-    if (    m_initialGraphicsSystem != graphicsSystem 
+    if (    m_initialGraphicsSystem != graphicsSystem
          && m_previousGraphicsSystem != graphicsSystem ) {
         KMessageBox::information (m_controlView->marbleWidget(),
                                 i18n("You have decided to run Marble with a different graphics system.\n"
                                    "For this change to become effective, Marble has to be restarted.\n"
                                    "Please close the application and start Marble again."),
                                 i18n("Graphics System Change") );
-    }    
+    }
     m_previousGraphicsSystem = graphicsSystem;
 
     // Time
@@ -1580,8 +1591,8 @@ void MarblePart::lockFloatItemPosition( bool enabled )
     QList<AbstractFloatItem *>::const_iterator i = floatItemList.constBegin();
     QList<AbstractFloatItem *>::const_iterator const end = floatItemList.constEnd();
     for (; i != end; ++i ) {
-        // Locking one would suffice as it affects all. 
-	// Nevertheless go through all.
+        // Locking one would suffice as it affects all.
+        // Nevertheless go through all.
         (*i)->setPositionLocked(enabled);
     }
 }
