@@ -23,7 +23,7 @@
 #include "AbstractProjection.h"
 #include "MarbleDirs.h"
 #include "MarbleDebug.h"
-#include "MarbleDataFacade.h"
+#include "MarbleModel.h"
 #include "ui_OverviewMapConfigWidget.h"
 
 #include "GeoPainter.h"
@@ -82,15 +82,11 @@ QString OverviewMap::description() const
 
 QIcon OverviewMap::icon () const
 {
-    return QIcon();
+    return QIcon(":/icons/worldmap.png");
 }
 
-QDialog *OverviewMap::configDialog() const
+QDialog *OverviewMap::configDialog()
 {
-    if ( MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen ) {
-        return false;
-    }
-
     if ( !m_configDialog ) {
         // Initializing configuration dialog
         m_configDialog = new QDialog();
@@ -106,8 +102,8 @@ QDialog *OverviewMap::configDialog() const
                  SLOT( writeSettings() ) );
         connect( ui_configWidget->m_buttonBox, SIGNAL( rejected() ),
                  SLOT( readSettings() ) );
-        connect( ui_configWidget->m_buttonBox, SIGNAL( clicked ( QAbstractButton * ) ),
-                 SLOT( evaluateClickedButton( QAbstractButton * ) ) );
+        connect( ui_configWidget->m_buttonBox->button( QDialogButtonBox::Reset ), SIGNAL( clicked () ),
+                 SLOT( restoreDefaultSettings() ) );
         QPushButton *applyButton = ui_configWidget->m_buttonBox->button( QDialogButtonBox::Apply );
         connect( applyButton, SIGNAL( clicked() ),
                  SLOT( writeSettings() ) );
@@ -139,9 +135,9 @@ bool OverviewMap::isInitialized () const
 void OverviewMap::changeViewport( ViewportParams *viewport )
 {
     GeoDataLatLonAltBox latLonAltBox = viewport->currentProjection()->latLonAltBox( QRect( QPoint( 0, 0 ), viewport->size() ), viewport );
-    qreal centerLon, centerLat;
-    viewport->centerCoordinates( centerLon, centerLat );
-    QString target = dataFacade()->target();
+    const qreal centerLon = viewport->centerLongitude();
+    const qreal centerLat = viewport->centerLatitude();
+    QString target = marbleModel()->planetId();
 
     if ( !( m_latLonAltBox == latLonAltBox
             && m_centerLon == centerLon
@@ -167,7 +163,7 @@ void OverviewMap::paintContent( GeoPainter *painter, ViewportParams *viewport,
 
     QRectF mapRect( contentRect() );
 
-    QString target = dataFacade()->target();
+    QString target = marbleModel()->planetId();
 
     if ( target != m_target ) {
         changeBackground( target );
@@ -229,8 +225,8 @@ void OverviewMap::paintContent( GeoPainter *painter, ViewportParams *viewport,
     qreal xSouth = mapRect.height() / 2.0
                     - mapRect.height() / M_PI * m_latLonAltBox.south();
 
-    qreal lon, lat;
-    viewport->centerCoordinates( lon, lat );
+    qreal lon = viewport->centerLongitude();
+    qreal lat = viewport->centerLatitude();
     GeoDataPoint::normalizeLonLat( lon, lat );
     qreal x = mapRect.width() / 2.0 + mapRect.width() / ( 2.0 * M_PI ) * lon;
     qreal y = mapRect.height() / 2.0 - mapRect.height() / M_PI * lat;
@@ -303,7 +299,9 @@ void OverviewMap::setSettings( QHash<QString,QVariant> settings )
     }
     foreach( const QString& planet, planets ) {
         if ( !settings.contains( "path_" + planet ) ) {
-            settings.insert( "path_" + planet, worldmap );
+            QString const planetMap = MarbleDirs::path( QString( "svg/%1map.svg" ).arg( planet ) );
+            QString const mapFile = planetMap.isEmpty() ? worldmap : planetMap;
+            settings.insert( "path_" + planet, mapFile );
         }
     }
 
@@ -316,7 +314,7 @@ void OverviewMap::setSettings( QHash<QString,QVariant> settings )
     emit settingsChanged( nameId() );
 }
 
-void OverviewMap::readSettings() const
+void OverviewMap::readSettings()
 {
     if ( !m_configDialog ) {
         return;
@@ -412,7 +410,7 @@ bool OverviewMap::eventFilter( QObject *object, QEvent *e )
     return AbstractFloatItem::eventFilter(object,e);
 }
 
-void OverviewMap::changeBackground( const QString& target ) const
+void OverviewMap::changeBackground( const QString& target )
 {
     m_svgobj.load( m_svgPaths[target] );
 }
@@ -422,7 +420,7 @@ QSvgWidget *OverviewMap::currentWidget() const
     return m_svgWidgets[m_planetID[ui_configWidget->m_planetComboBox->currentIndex()]];
 }
 
-void OverviewMap::setCurrentWidget( QSvgWidget *widget ) const
+void OverviewMap::setCurrentWidget( QSvgWidget *widget )
 {
     m_svgWidgets[m_planetID[ui_configWidget->m_planetComboBox->currentIndex()]] = widget;
     m_mapChanged = true;
@@ -431,7 +429,7 @@ void OverviewMap::setCurrentWidget( QSvgWidget *widget ) const
     }
 }
 
-void OverviewMap::loadPlanetMaps() const
+void OverviewMap::loadPlanetMaps()
 {
     foreach( const QString& planet, m_planetID ) {
         if ( m_svgWidgets.contains( planet) ) {
@@ -442,7 +440,7 @@ void OverviewMap::loadPlanetMaps() const
     }
 }
 
-void OverviewMap::loadMapSuggestions() const
+void OverviewMap::loadMapSuggestions()
 {
     QStringList paths = QDir( MarbleDirs::pluginPath( "" ) ).entryList( QStringList( "*.svg" ), QDir::Files | QDir::NoDotAndDotDot );
     for( int i = 0; i < paths.size(); ++i ) {
@@ -511,12 +509,6 @@ void OverviewMap::useMapSuggestion( int index )
     showCurrentPlanetPreview();
 }
 
-void OverviewMap::evaluateClickedButton( QAbstractButton *button )
-{
-    if( button == ui_configWidget->m_buttonBox->button( QDialogButtonBox::Reset ) )
-        restoreDefaultSettings();
-}
-
 }
 
 Q_EXPORT_PLUGIN2( OverviewMap, Marble::OverviewMap )
