@@ -19,7 +19,6 @@
 #include "MarbleDirs.h"
 #include "MarbleDebug.h"
 #include "GeoDataLatLonBox.h"
-#include "GeoDataParser.h"
 #include "GeoDataDocument.h"
 #include "GeoDataData.h"
 #include "GeoDataExtendedData.h"
@@ -253,7 +252,7 @@ MarbleAbstractRunner* MonavPlugin::newRunner() const
     return new MonavRunner( this );
 }
 
-QString MonavPlugin::mapDirectoryForRequest( RouteRequest* request ) const
+QString MonavPlugin::mapDirectoryForRequest( const RouteRequest* request ) const
 {
     d->initialize();
 
@@ -287,6 +286,35 @@ QString MonavPlugin::mapDirectoryForRequest( RouteRequest* request ) const
     return QString();
 }
 
+QStringList MonavPlugin::mapDirectoriesForRequest( const RouteRequest* request ) const
+{
+    QStringList result;
+    d->initialize();
+    QHash<QString, QVariant> settings = request->routingProfile().pluginSettings()[nameId()];
+    QString transport = settings["transport"].toString();
+
+    for ( int j=0; j<d->m_maps.size(); ++j ) {
+        bool valid = true;
+        if ( transport.isEmpty() || transport == d->m_maps[j].transport() ) {
+            for ( int i = 0; i < request->size(); ++i ) {
+                GeoDataCoordinates via = request->at( i );
+                if ( !d->m_maps[j].containsPoint( via ) ) {
+                    valid = false;
+                    break;
+                }
+            }
+        } else {
+            valid = false;
+        }
+
+        if ( valid ) {
+            result << d->m_maps[j].directory().absolutePath();
+        }
+    }
+
+    return result;
+}
+
 RunnerPlugin::ConfigWidget *MonavPlugin::configWidget()
 {
     return new MonavConfigWidget( this );
