@@ -36,9 +36,11 @@ public:
 
     MonavRunnerPrivate( const MonavPlugin* plugin );
 
-    bool retrieveData( RouteRequest *route, RoutingResult* result ) const;
+    bool retrieveData( const RouteRequest *route, RoutingResult* result ) const;
 
-    GeoDataLineString* retrieveRoute( RouteRequest *route, QVector<GeoDataPlacemark*> *instructions ) const;
+    bool retrieveData( const RouteRequest *route, const QString &mapDir, RoutingResult* result ) const;
+
+    GeoDataLineString* retrieveRoute( const RouteRequest *route, QVector<GeoDataPlacemark*> *instructions ) const;
 
     GeoDataDocument* createDocument( GeoDataLineString* geometry, const QVector<GeoDataPlacemark*> &instructions  ) const;
 };
@@ -49,13 +51,33 @@ MonavRunnerPrivate::MonavRunnerPrivate( const MonavPlugin* plugin ) :
     // nothing to do
 }
 
-bool MonavRunnerPrivate::retrieveData( RouteRequest *route, RoutingResult* reply ) const
+bool MonavRunnerPrivate::retrieveData( const RouteRequest *route, RoutingResult* reply ) const
 {
     QString mapDir = m_plugin->mapDirectoryForRequest( route );
     if ( mapDir.isEmpty() ) {
         return false;
     }
 
+    if ( retrieveData( route, mapDir, reply ) ) {
+        return true;
+    }
+
+    // The separation into two different methods to determine a first country candidate
+    // and a list of alternative ones if the first candidate fails is intentional
+    // for performance reasons. Do not merge both.
+    QStringList alternatives = m_plugin->mapDirectoriesForRequest( route );
+    alternatives.removeOne( mapDir );
+    foreach( const QString &mapDir, alternatives ) {
+        if ( retrieveData( route, mapDir, reply ) ) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
+bool MonavRunnerPrivate::retrieveData( const RouteRequest *route, const QString &mapDir, RoutingResult* reply ) const
+{
     QLocalSocket socket;
     socket.connectToServer( "MoNavD" );
     if ( socket.waitForConnected() ) {
@@ -114,7 +136,7 @@ bool MonavRunnerPrivate::retrieveData( RouteRequest *route, RoutingResult* reply
     return false;
 }
 
-GeoDataLineString* MonavRunnerPrivate::retrieveRoute( RouteRequest *route, QVector<GeoDataPlacemark*> *instructions ) const
+GeoDataLineString* MonavRunnerPrivate::retrieveRoute( const RouteRequest *route, QVector<GeoDataPlacemark*> *instructions ) const
 {
     GeoDataLineString* geometry = new GeoDataLineString;
     RoutingResult reply;
@@ -217,7 +239,7 @@ GeoDataFeature::GeoDataVisualCategory MonavRunner::category() const
     return GeoDataFeature::OsmSite;
 }
 
-void MonavRunner::retrieveRoute( RouteRequest *route )
+void MonavRunner::retrieveRoute( const RouteRequest *route )
 {
     QVector<GeoDataPlacemark*> instructions;
     GeoDataLineString* waypoints = d->retrieveRoute( route, &instructions );
